@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
@@ -72,6 +72,7 @@
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 
 struct _AnjutaProperty
 {
@@ -80,6 +81,7 @@ struct _AnjutaProperty
 	gchar                    *default_value;
 	guint                     flags;
 	gint                     notify_id;
+	GConfClient              *gclient;
 	
 	/* Set true if custom set/get to be used */
 	gboolean                  custom;
@@ -96,9 +98,20 @@ struct _AnjutaProperty
 
 struct _AnjutaPreferencesPriv
 {
-	GConfClient *gclient;
-	GList       *properties;
-	gboolean     is_showing;
+	GConfClient         *gclient;
+	GHashTable          *properties;
+	GtkWidget           *prefs_dialog;
+	AnjutaPluginManager *plugin_manager;
+	gboolean             is_showing;
+};
+
+/* Internal structure for anjuta_preferences_foreach */
+struct _AnjutaPreferencesForeachData
+{
+	AnjutaPreferences *pr;
+	AnjutaPreferencesFilterType filter;
+	AnjutaPreferencesCallback callback;
+	gpointer callback_data;
 };
 
 #define PREFERENCE_PROPERTY_PREFIX "preferences_"
@@ -465,22 +478,14 @@ anjuta_preferences_set_int (AnjutaPreferences *pr, const gchar *key,
 	}
 }
 
-static void
-on_preferences_dialog_response (GtkDialog *dialog,
-                                gint response, gpointer user_data)
-{
-	AnjutaPreferences *pr = ANJUTA_PREFERENCES (user_data);
-	if (pr)
-		gtk_widget_hide (GTK_WIDGET (pr));
-}
-
 static void
 property_destroy (AnjutaProperty *property)
 {
 	g_return_if_fail (property);
 	if (property->key) g_free (property->key);
-	if (property->default_value) g_free (property->key);
-	g_object_unref (G_OBJECT (property->object));
+	if (property->default_value) g_free (property->default_value);
+	g_object_unref (property->object);
+	gconf_client_notify_remove (property->gclient, property->notify_id);
 	g_free (property);
 }
 
@@ -503,8 +508,8 @@ get_object_type_from_string (const gchar* object_type)
 {
 	if (strcmp (object_type, "entry") == 0)
 		return ANJUTA_PROPERTY_OBJECT_TYPE_ENTRY;
-	else if (strcmp (object_type, "menu") == 0)
-		return ANJUTA_PROPERTY_OBJECT_TYPE_MENU;
+	else if (strcmp (object_type, "combo") == 0)
+		return ANJUTA_PROPERTY_OBJECT_TYPE_COMBO;
 	else if (strcmp (object_type, "spin") == 0)
 		return ANJUTA_PROPERTY_OBJECT_TYPE_SPIN;
 	else if (strcmp (object_type, "toggle") == 0)
@@ -515,6 +520,8 @@ get_object_type_from_string (const gchar* object_type)
 		return ANJUTA_PROPERTY_OBJECT_TYPE_COLOR;
 	else if (strcmp (object_type, "font") == 0)
 		return ANJUTA_PROPERTY_OBJECT_TYPE_FONT;
+	else if (strcmp (object_type, "file") == 0)
+		return ANJUTA_PROPERTY_OBJECT_TYPE_FILE;
 	else
 		return (AnjutaPropertyObjectType)(-1);
 }
@@ -572,11 +579,11 @@ get_property_value_as_string (AnjutaProperty *prop)
 		text_value =
 			gtk_editable_get_chars (GTK_EDITABLE (prop->object), 0, -1);
 		break;
-	case ANJUTA_PROPERTY_OBJECT_TYPE_MENU:
+	case ANJUTA_PROPERTY_OBJECT_TYPE_COMBO:
 		{
 			gint idx;
 			values = g_object_get_data(G_OBJECT(prop->object), "untranslated");
-			idx = gtk_option_menu_get_history(GTK_OPTION_MENU(prop->object));
+			idx = gtk_combo_box_get_active(GTK_COMBO_BOX(prop->object));
 			if (values[idx] != NULL)
 				text_value = g_strdup(values[idx]);
 			break;
@@ -609,6 +616,13 @@ get_property_value_as_string (AnjutaProperty *prop)
 			text_value = g_strdup (font);
 		}
 		break;
+	case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
+		{
+			const gchar *filename;
+			filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (prop->object));
+			text_value = g_strdup (filename);
+		}
+		break;
 	}
 	if (text_value && (strlen (text_value) == 0))
 	{
@@ -677,7 +691,7 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 		else
 			gtk_entry_set_text (GTK_ENTRY (prop->object), "");
 		break;
-	case ANJUTA_PROPERTY_OBJECT_TYPE_MENU:
+	case ANJUTA_PROPERTY_OBJECT_TYPE_COMBO:
 		values = g_object_get_data(G_OBJECT(prop->object), "untranslated");
 		if (value != NULL)
 		{
@@ -685,7 +699,7 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 			{
 				if (strcmp(value, values[i]) == 0)
 				{
-					gtk_option_menu_set_history(GTK_OPTION_MENU(prop->object), i);
+					gtk_combo_box_set_active(GTK_COMBO_BOX(prop->object), i);
 					break;
 				}
 			}
@@ -752,6 +766,20 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 											 "A standard font");
 		}*/
 		break;
+		
+	case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
+		{	
+			if (value)
+			{
+				gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (prop->object),
+																 value);
+			}
+			else
+			{
+				gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (prop->object), "");
+			}
+		}
+		break;
 	}
 }
 
@@ -861,6 +889,23 @@ update_property_on_change_font (GtkWidget *widget,
 	g_free (val);
 }
 
+static void
+unregister_preferences_key (GtkWidget *widget,
+							gpointer user_data)
+{
+	AnjutaProperty *p;
+	AnjutaPreferences *pr;
+	gchar *key;
+	
+	p = (AnjutaProperty *) user_data;
+	pr = g_object_get_data (G_OBJECT (widget),
+							"AnjutaPreferences");
+	key = g_strdup (p->key);
+	
+	g_hash_table_remove (pr->priv->properties, key);
+	g_free (key);
+}
+
 static void
 get_property (GConfClient *gclient, guint cnxt_id,
 			  GConfEntry *entry, gpointer user_data)
@@ -926,7 +971,7 @@ register_callbacks (AnjutaPreferences *pr, AnjutaProperty *p)
 			g_signal_connect (G_OBJECT(p->object), "focus_out_event",
 							  G_CALLBACK (update_property_on_event_str), p);
 			break;
-		case ANJUTA_PROPERTY_OBJECT_TYPE_MENU:
+		case ANJUTA_PROPERTY_OBJECT_TYPE_COMBO:
 			g_signal_connect (G_OBJECT(p->object), "changed",
 							  G_CALLBACK (update_property_on_change_str), p);
 			break;
@@ -938,6 +983,10 @@ register_callbacks (AnjutaPreferences *pr, AnjutaProperty *p)
 			g_signal_connect (G_OBJECT(p->object), "color-set",
 							  G_CALLBACK (update_property_on_change_color), p);
 			break;
+		case ANJUTA_PROPERTY_OBJECT_TYPE_FILE:
+			g_signal_connect (G_OBJECT(p->object), "file-set",
+							  G_CALLBACK (update_property_on_change_str), p);
+			break;
 		default:
 			break;
 	}
@@ -949,6 +998,49 @@ register_callbacks (AnjutaPreferences *pr, AnjutaProperty *p)
 	}
 	p->notify_id = gconf_client_notify_add (gclient, build_key (p->key),
 											get_property, p, NULL, NULL);
+	
+	/* Connect to widget destroy signal so we can automatically unregister
+	 * keys so there aren't any potential conflicts or references to 
+	 * nonexistent widgets on subsequent uses of the prefs dialog. */
+	g_signal_connect (G_OBJECT (p->object), "destroy",
+					  G_CALLBACK (unregister_preferences_key),
+					  p);
+}
+
+static gboolean
+preferences_foreach_callback (gchar *key, struct _AnjutaProperty *p, 
+							  struct _AnjutaPreferencesForeachData *data)
+{	
+	if (p->object_type != ANJUTA_PROPERTY_OBJECT_TYPE_COMBO)
+	{
+		if (data->filter == ANJUTA_PREFERENCES_FILTER_NONE)
+			return data->callback (data->pr, key, data->callback_data);
+		else if (p->flags & data->filter)
+			return data->callback (data->pr, key, data->callback_data);
+	}
+	
+	return TRUE;
+}
+
+static void
+connect_prop_to_object (AnjutaPreferences *pr, AnjutaProperty *p)
+{
+	int gconf_value;
+	gchar *value;
+	
+	if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL ||
+		p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
+	{	
+		gconf_value = anjuta_preferences_get_int (pr, p->key);
+		value = g_strdup_printf ("%d", gconf_value);
+		set_property_value_as_string (p, value);
+	} 
+	else 
+	{
+		value = anjuta_preferences_get (pr, p->key);
+		set_property_value_as_string (p, value);
+		g_free (value);
+	}
 }
 
 
@@ -991,6 +1083,7 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 	p->object_type = object_type;
 	p->data_type = data_type;
 	p->key = g_strdup (key);
+	p->gclient = pr->priv->gclient;
 	
 	value = gconf_client_get (pr->priv->gclient,
 							  build_key (p->key), NULL);
@@ -1014,8 +1107,8 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 		p->default_value = g_strdup (default_value);
 		if (strlen (default_value) > 0)
 		{
-			/* For menu, initialize the untranslated strings */
-			if (object_type == ANJUTA_PROPERTY_OBJECT_TYPE_MENU) 
+			/* For combo, initialize the untranslated strings */
+			if (object_type == ANJUTA_PROPERTY_OBJECT_TYPE_COMBO) 
 			{
 				gchar *old_value;
 				gchar **vstr;
@@ -1073,7 +1166,9 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 	p->custom = FALSE;
 	p->set_property = NULL;
 	p->get_property = NULL;
-	pr->priv->properties = g_list_prepend (pr->priv->properties, p);
+	
+	g_hash_table_insert (pr->priv->properties, g_strdup (key), p);
+	connect_prop_to_object (pr, p);
 	register_callbacks (pr, p);
 	return TRUE;
 }
@@ -1119,6 +1214,7 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
 	p->object_type = (AnjutaPropertyObjectType) 0;
 	p->data_type = data_type;
 	p->key = g_strdup (key);
+	p->gclient = pr->priv->gclient;
 	
 	value = gconf_client_get (pr->priv->gclient,
 							  build_key (p->key), NULL);
@@ -1179,8 +1275,8 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
 	p->flags = flags;
 	p->set_property = set_property;
 	p->get_property = get_property;
-	
-	pr->priv->properties = g_list_prepend (pr->priv->properties, p);
+
+	g_hash_table_insert (pr->priv->properties, g_strdup (key), p);
 	return TRUE;
 }
 
@@ -1352,64 +1448,17 @@ anjuta_preferences_foreach (AnjutaPreferences *pr,
 							AnjutaPreferencesCallback callback,
 							gpointer data)
 {
-	AnjutaProperty *p;
-	GList *node;
-	gboolean go_on = TRUE;
+	struct _AnjutaPreferencesForeachData foreach_data;
 	
-	g_return_if_fail (ANJUTA_IS_PREFERENCES (pr));
-	g_return_if_fail (callback != NULL);
+	foreach_data.pr = pr;
+	foreach_data.filter = filter;
+	foreach_data.callback = callback;
+	foreach_data.callback_data = data;
 	
-	node = pr->priv->properties;
-	while (node && go_on)
-	{
-		p = node->data;
-		if (p->object_type != ANJUTA_PROPERTY_OBJECT_TYPE_MENU)
-		{
-			if (filter == ANJUTA_PREFERENCES_FILTER_NONE)
-				go_on = callback (pr, p->key, data);
-			else if (p->flags & filter)
-				go_on = callback (pr, p->key, data);
-		}
-		node = g_list_next (node);
-	}
-}
+	g_hash_table_find (pr->priv->properties, 
+					   (GHRFunc) preferences_foreach_callback,
+					   &foreach_data);
 
-static gboolean
-on_preferences_dialog_delete_event (GtkDialog *dialog,
-									GdkEvent *event,
-									gpointer user_data)
-{
-	gtk_widget_hide (GTK_WIDGET(dialog));
-	return TRUE;
-}
-
-static gboolean
-preferences_prop_to_objects (AnjutaPreferences *pr)
-{
-	AnjutaProperty *p;
-	GList *node;
-	node = pr->priv->properties;
-	while (node)
-	{
-		p = node->data;
-		if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL ||
-			p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
-		{
-			int gconf_value;
-			gchar *value;
-			
-			gconf_value = anjuta_preferences_get_int (pr, p->key);
-			value = g_strdup_printf ("%d", gconf_value);
-			set_property_value_as_string (p, value);
-		} else {
-			gchar *value;
-			value = anjuta_preferences_get (pr, p->key);
-			set_property_value_as_string (p, value);
-			g_free (value);
-		}
-		node = g_list_next (node);
-	}
-	return TRUE;
 }
 
 /**
@@ -1431,7 +1480,7 @@ preferences_prop_to_objects (AnjutaPreferences *pr)
  * <programlisting>
  *     preferences_OBJECTTYPE:DATATYPE:DEFAULT:FLAGS:PROPERTYKEY
  *     where,
- *       OBJECTTYPE is 'toggle', 'spin', 'entry', 'text', 'color' or 'font'.
+ *       OBJECTTYPE is 'toggle', 'spin', 'entry', 'text', 'color', 'font' or 'file' .
  *       DATATYPE   is 'bool', 'int', 'float', 'text', 'color' or 'font'.
  *       DEFAULT    is the default value (in the appropriate format). The format
  *                     for color is '#XXXXXX' representing RGB value and for
@@ -1483,7 +1532,7 @@ anjuta_preferences_add_page (AnjutaPreferences* pr, GladeXML *gxml,
 	}
 	image_path = anjuta_res_get_pixmap_file (icon_filename);
 	pixbuf = gdk_pixbuf_new_from_file (image_path, NULL);
-	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (pr),
+	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (pr->priv->prefs_dialog),
 										glade_widget_name, title, pixbuf, page);
 	anjuta_preferences_register_all_properties_from_glade_xml (pr, gxml, page);
 	g_object_unref (page);
@@ -1491,11 +1540,84 @@ anjuta_preferences_add_page (AnjutaPreferences* pr, GladeXML *gxml,
 	g_object_unref (pixbuf);
 }
 
+void
+anjuta_preferences_remove_page (AnjutaPreferences *pr,
+								const gchar *page_name)
+{
+	if (pr->priv->prefs_dialog)
+	{
+		anjuta_preferences_dialog_remove_page (ANJUTA_PREFERENCES_DIALOG (pr->priv->prefs_dialog),
+											   page_name);
+	}
+}
+
+static void 
+on_preferences_dialog_destroyed (GtkWidget *preferencess_dialog,
+							     AnjutaPreferences *pr)
+{
+	GList *plugins;
+	GList *current_plugin;
+	
+	plugins = anjuta_plugin_manager_get_active_plugin_objects (pr->priv->plugin_manager);
+	current_plugin = plugins;
+	
+	while (current_plugin)
+	{
+		if (IANJUTA_IS_PREFERENCES (current_plugin->data))
+		{
+			ianjuta_preferences_unmerge (IANJUTA_PREFERENCES (current_plugin->data),
+								  		 pr, NULL);
+		}
+			
+		current_plugin = g_list_next (current_plugin);
+	
+	}
+	
+	g_object_unref (pr->priv->prefs_dialog);
+	
+	g_list_free (plugins);
+	pr->priv->prefs_dialog = NULL;
+}
+
+GtkWidget *
+anjuta_preferences_get_dialog (AnjutaPreferences *pr)
+{
+	GList *plugins;
+	GList *current_plugin;
+	
+	if (pr->priv->prefs_dialog)
+		return pr->priv->prefs_dialog;
+	else
+	{
+		pr->priv->prefs_dialog = anjuta_preferences_dialog_new ();
+		
+		g_signal_connect (G_OBJECT (pr->priv->prefs_dialog), "destroy",
+						  G_CALLBACK (on_preferences_dialog_destroyed),
+						  pr);
+
+		plugins = anjuta_plugin_manager_get_active_plugin_objects (pr->priv->plugin_manager);
+		current_plugin = plugins;
+		
+		while (current_plugin)
+		{
+			if (IANJUTA_IS_PREFERENCES (current_plugin->data))
+			{
+				ianjuta_preferences_merge (IANJUTA_PREFERENCES (current_plugin->data),
+								  		   pr, NULL);
+			}
+			
+			current_plugin = g_list_next (current_plugin);
+		}
+	
+		g_list_free (plugins);
+		
+		return g_object_ref_sink (pr->priv->prefs_dialog);
+	}
+}
+
 static void anjuta_preferences_class_init    (AnjutaPreferencesClass *class);
 static void anjuta_preferences_instance_init (AnjutaPreferences      *pr);
 
-static AnjutaPreferencesDialogClass *parent_class;
-
 GType
 anjuta_preferences_get_type ()
 {
@@ -1516,7 +1638,7 @@ anjuta_preferences_get_type ()
 			(GInstanceInitFunc) anjuta_preferences_instance_init,
 			NULL            /* value_table */
 		};
-		obj_type = g_type_register_static (ANJUTA_TYPE_PREFERENCES_DIALOG,
+		obj_type = g_type_register_static (G_TYPE_OBJECT,
 		                                   "AnjutaPreferences", &obj_info, 0);
 	}
 	return obj_type;
@@ -1526,88 +1648,49 @@ static void
 anjuta_preferences_dispose (GObject *obj)
 {
 	AnjutaPreferences *pr = ANJUTA_PREFERENCES (obj);
+	
 	if (pr->priv->properties)
 	{
 		/* This will release the refs on property objects */
-		g_list_foreach (pr->priv->properties, (GFunc)property_destroy, NULL);
+		g_hash_table_destroy (pr->priv->properties);
 		pr->priv->properties = NULL;
 	}
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
 }
 
 static void
 anjuta_preferences_instance_init (AnjutaPreferences *pr)
 {
-	pr->priv = g_new0(AnjutaPreferencesPriv, 1);
-	pr->priv->properties = NULL;
+	pr->priv = g_new0 (AnjutaPreferencesPriv, 1);
 	
-	/* DEBUG_PRINT ("Initializing AP Instance"); */
+	pr->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+												  g_free, 
+												  (GDestroyNotify) property_destroy);
 	
-	pr->priv->is_showing = FALSE;
 	pr->priv->gclient = gconf_client_get_default();
 	gconf_client_add_dir (pr->priv->gclient, GCONF_KEY_PREFIX,
 						  GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	/* Add buttons: Close button */
-	gtk_dialog_add_button (GTK_DIALOG (pr),
-				       GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL);
-	g_signal_connect (G_OBJECT (pr), "delete_event",
-					  G_CALLBACK (on_preferences_dialog_delete_event), pr);
-	g_signal_connect (G_OBJECT (pr), "response",
-					  G_CALLBACK (on_preferences_dialog_response), pr);
+
 }
 
 static void
 anjuta_preferences_finalize (GObject *obj)
 {
 	AnjutaPreferences *pr = ANJUTA_PREFERENCES (obj);
-
+	
+	if (pr->priv->prefs_dialog)
+		gtk_widget_destroy (pr->priv->prefs_dialog);
+	
+	g_object_unref (pr->priv->plugin_manager);
 	g_free (pr->priv);
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
-}
-
-static void
-anjuta_preferences_hide (GtkWidget *w)
-{
-	AnjutaPreferences *pr = ANJUTA_PREFERENCES (w);	
-	pr->priv->is_showing = FALSE;
-	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, hide, (w));
-}
-
-static void
-anjuta_preferences_show (GtkWidget * w)
-{
-	AnjutaPreferences *pr = ANJUTA_PREFERENCES (w);
-	if (pr->priv->is_showing)
-		return;
-	preferences_prop_to_objects (ANJUTA_PREFERENCES (pr));
-	pr->priv->is_showing = TRUE;
-	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, show, (w));
-}
-
-static void
-anjuta_preferences_close (GtkDialog *obj)
-{
-	AnjutaPreferences *pr = ANJUTA_PREFERENCES (obj);
-	gtk_widget_hide (GTK_WIDGET (pr));
 }
 
 static void
 anjuta_preferences_class_init (AnjutaPreferencesClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
-	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
-	GtkDialogClass *dialog_class = GTK_DIALOG_CLASS (class);
-	
-	parent_class = g_type_class_peek_parent (class);
 	
 	object_class->dispose = anjuta_preferences_dispose;
 	object_class->finalize = anjuta_preferences_finalize;
-
-	widget_class->hide = anjuta_preferences_hide;
-	widget_class->show = anjuta_preferences_show;
-	
-	// dialog_class->response = anjuta_preferences_response;
-	dialog_class->close = anjuta_preferences_close;
 }
 
 /**
@@ -1617,17 +1700,16 @@ anjuta_preferences_class_init (AnjutaPreferencesClass *class)
  * 
  * Return value: A #AnjutaPreferences object.
  */
-GtkWidget *
-anjuta_preferences_new ()
+AnjutaPreferences *
+anjuta_preferences_new (AnjutaPluginManager *plugin_manager)
 {
-	static GtkWidget *widget = NULL;
-
-	if (!widget) {
-		widget = gtk_widget_new (ANJUTA_TYPE_PREFERENCES,
-					 "title", _("Anjuta Preferences"),
-					 NULL);
-	}
-	return widget;
+	AnjutaPreferences *pr;
+	
+	pr = g_object_new (ANJUTA_TYPE_PREFERENCES, NULL);
+	pr->priv->plugin_manager = g_object_ref (plugin_manager);
+	
+	return pr;
+	
 }
 
 /**
