@@ -6,6 +6,8 @@
 #include "ianjuta-cpu-debugger.h"
 #include "ianjuta-debug-manager.h"
 #include "ianjuta-debugger.h"
+#include "ianjuta-debugger-breakpoint.h"
+#include "ianjuta-document.h"
 #include "ianjuta-document-manager.h"
 #include "ianjuta-editor.h"
 #include "ianjuta-editor-assist.h"
@@ -19,6 +21,7 @@
 #include "ianjuta-editor-hover.h"
 #include "ianjuta-editor-language.h"
 #include "ianjuta-editor-line-mode.h"
+#include "ianjuta-editor-search.h"
 #include "ianjuta-editor-selection.h"
 #include "ianjuta-editor-view.h"
 #include "ianjuta-editor-zoom.h"
@@ -30,12 +33,14 @@
 #include "ianjuta-indicable.h"
 #include "ianjuta-iterable.h"
 #include "ianjuta-iterable-tree.h"
+#include "ianjuta-language.h"
 #include "ianjuta-language-support.h"
 #include "ianjuta-loader.h"
 #include "ianjuta-macro.h"
 #include "ianjuta-markable.h"
 #include "ianjuta-message-manager.h"
 #include "ianjuta-message-view.h"
+#include "ianjuta-plugin-factory.h"
 #include "ianjuta-preferences.h"
 #include "ianjuta-print.h"
 #include "ianjuta-project-manager.h"
