@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -30,6 +30,7 @@
 
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-utils.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
 #include <libanjuta/interfaces/ianjuta-terminal.h>
@@ -37,6 +38,9 @@
 
 #include "executer.h"
 
+#define PREF_USE_SB "build.use_scratchbox"
+#define PREF_SB_PATH "build.scratchbox.path"
+
 static gboolean
 get_program_parameters (BasicAutotoolsPlugin *plugin,
 						const gchar *pre_select_uri,
@@ -67,7 +71,11 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 							 IANJUTA_PROJECT_MANAGER_TARGET_EXECUTABLE,
 											 NULL);
 		if (!exec_targets)
+		{
+			anjuta_util_dialog_error(GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell),
+															 _("No executables in this project!"));
 			return FALSE;
+		}
 	}
 	else
 	{
@@ -88,14 +96,18 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 	if (plugin->program_args)
 		gtk_entry_set_text (GTK_ENTRY (arguments_entry),
 							plugin->program_args);
-		
+	
 	if (g_list_length (exec_targets) > 0)
 	{
 		/* Populate treeview */
-		gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW
-																  (treeview)),
+    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+ 		gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
+								 GTK_TREE_MODEL (store));
+		g_object_unref (store);
+		GtkTreeSelection* selection =
+          gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
+    gtk_tree_selection_set_mode (selection,
 									 GTK_SELECTION_BROWSE);
-		store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 		node = exec_targets;
 		while (node)
 		{
@@ -117,16 +129,25 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter, 0, rel_path, 1,
 								node->data, -1);
+      
+      if (plugin->last_exec_uri && g_str_equal (plugin->last_exec_uri, node->data))
+      { 
+				GtkTreePath* path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
+        gtk_tree_selection_select_iter (selection, &iter);
+
+				gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (treeview), path, NULL, FALSE, 0, 0);
+				
+				gtk_tree_path_free (path);
+        g_free (plugin->last_exec_uri);
+        plugin->last_exec_uri = NULL;
+      }
+      
 			g_free (local_path);
 			g_free (node->data);
 			node = g_list_next (node);
 		}
 		g_list_free (exec_targets);
 		
-		gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
-								 GTK_TREE_MODEL (store));
-		g_object_unref (store);
-		
 		column = gtk_tree_view_column_new ();
 		gtk_tree_view_column_set_sizing (column,
 										 GTK_TREE_VIEW_COLUMN_AUTOSIZE);
@@ -138,6 +159,12 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 											0);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 		gtk_tree_view_set_expander_column (GTK_TREE_VIEW (treeview), column);
+                   
+    if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+    {
+      gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
+      gtk_tree_selection_select_iter (selection, &iter);
+    }
 	}
 	else
 	{
@@ -146,7 +173,8 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 	}
 	
 	/* Run dialog */
-	response = gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_dialog_set_default_response (GTK_DIALOG(dlg), GTK_RESPONSE_OK);
+  response = gtk_dialog_run (GTK_DIALOG (dlg));
 	if (response == GTK_RESPONSE_OK)
 	{
 		GtkTreeSelection *sel;
@@ -160,13 +188,15 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 			{
 				gtk_tree_model_get (model, &iter, 1, &target, -1);
 				if (program_uri)
-					*program_uri = g_strdup (target);
+					*program_uri = target;
 				if (run_in_terminal)
 					*run_in_terminal = gtk_toggle_button_get_active (
 									GTK_TOGGLE_BUTTON (use_terminal_check));
 				if (program_args)
 					*program_args = g_strdup (gtk_entry_get_text (
 											GTK_ENTRY (arguments_entry)));
+
+        plugin->last_exec_uri = g_strdup(target);
 				success = TRUE;
 			}
 		}
@@ -189,6 +219,7 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 void
 execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 {
+	AnjutaPreferences* prefs = anjuta_shell_get_preferences (ANJUTA_PLUGIN(plugin)->shell, NULL);
 	gboolean run_in_terminal, error_condition;
 	gchar *target = NULL, *args = NULL, *dir = NULL, *cmd = NULL;
 
@@ -335,14 +366,25 @@ execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 		return;
 	}
 	
+	
 	if (args && strlen (args) > 0)
 		cmd = g_strconcat (target, " ", args, NULL);
 	else
 		cmd = g_strdup (target);
-	
-	DEBUG_PRINT ("Command is: %s", cmd);
 
-	dir = g_path_get_dirname (target);
+	if (anjuta_preferences_get_int (prefs , PREF_USE_SB))
+	{
+		const gchar* sb_path = anjuta_preferences_get(prefs, PREF_SB_PATH);
+		/* we need to skip the /scratchbox/users part, maybe could be done more clever */
+		const gchar* real_dir = strstr(target, "/home");
+		gchar* oldcmd = cmd;
+		cmd = g_strdup_printf("%s/login -d %s \"%s\"", sb_path,
+									  real_dir, oldcmd);
+		g_free(oldcmd);
+		dir = g_strdup(real_dir);
+	}
+	else
+		dir = g_path_get_dirname (target);
 	if (run_in_terminal)
 	{
 		IAnjutaTerminal *term;
@@ -351,7 +393,7 @@ execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 										   IAnjutaTerminal, NULL);
 		if (term)
 		{
-			ianjuta_terminal_execute_command (term, "", cmd, NULL);
+			ianjuta_terminal_execute_command (term, dir, cmd, NULL);
 		}
 		else
 		{
