@@ -31,7 +31,7 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-message-manager.h>
 #include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
-#include <libanjuta/interfaces/ianjuta-breakpoint-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
 #include <libanjuta/interfaces/ianjuta-variable-debugger.h>
 
 
@@ -50,7 +50,7 @@ struct _DmaDebuggerQueue {
 
 	AnjutaPlugin* plugin;
 	IAnjutaDebugger* debugger;
-	gint feature;
+	guint support;
 
 	/* Command queue */
 	GQueue *queue;
@@ -157,13 +157,43 @@ dma_debugger_queue_clear (DmaDebuggerQueue *self)
 static void
 dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState state, GError* err)
 {
-	IAnjutaDebuggerState signal = IANJUTA_DEBUGGER_BUSY;
-
-	DEBUG_PRINT("update debugger state new %d old %d", state, self->debugger_state);
+	enum
+	{
+		NO_SIGNAL,
+		DEBUGGER_STOPPED_SIGNAL,
+		DEBUGGER_STARTED_SIGNAL,
+		PROGRAM_LOADED_SIGNAL,
+		PROGRAM_UNLOADED_SIGNAL,
+		PROGRAM_STARTED_SIGNAL,
+		PROGRAM_EXITED_SIGNAL,
+		PROGRAM_RUNNING_SIGNAL,
+		PROGRAM_STOPPED_SIGNAL
+	} signal = NO_SIGNAL;
+	gboolean emit_program_started = FALSE;
 	
+	DEBUG_PRINT("update debugger state new %d old %d", state, self->debugger_state);
+
 	/* Add missing state if useful */
 	switch (state)
 	{
+	case IANJUTA_DEBUGGER_STOPPED:
+		if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
+			(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
+		{
+			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
+		}
+		else if	(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_LOADED)
+		{
+			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
+		}
+		break;
+	case IANJUTA_DEBUGGER_STARTED:
+		if ((self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_RUNNING) ||
+			(self->debugger_state == IANJUTA_DEBUGGER_PROGRAM_STOPPED))
+		{
+			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
+		}
+		break;
 	case IANJUTA_DEBUGGER_PROGRAM_LOADED:
 		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
 		{
@@ -171,71 +201,120 @@ dma_queue_emit_debugger_state (DmaDebuggerQueue *self, IAnjutaDebuggerState stat
 		}
 		break;
 	case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
+		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
+		{
+			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
+		}
+		else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
+		{
+			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
+		}
+		break;
 	case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
 		if (self->debugger_state == IANJUTA_DEBUGGER_STOPPED)
 		{
 			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_STARTED, NULL);
 		}
-		if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
+		else if (self->debugger_state == IANJUTA_DEBUGGER_STARTED)
 		{
 			dma_queue_emit_debugger_state (self, IANJUTA_DEBUGGER_PROGRAM_LOADED, NULL);
 		}
 		break;
 	case IANJUTA_DEBUGGER_BUSY:
-	case IANJUTA_DEBUGGER_STOPPED:
-	case IANJUTA_DEBUGGER_STARTED:
-		/* Nothing to do */
 		break;
 	}	
 
-	switch (state)
+	if (self->debugger_state != state)
 	{
-	case IANJUTA_DEBUGGER_BUSY:
-		/* Debugger is busy, nothing to do */
-		return;
-	case IANJUTA_DEBUGGER_STOPPED:
-	case IANJUTA_DEBUGGER_STARTED:
-	case IANJUTA_DEBUGGER_PROGRAM_LOADED:
-	case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
-		if (self->debugger_state != state) 
+		switch (state)
 		{
-			self->debugger_state = state;
+		case IANJUTA_DEBUGGER_BUSY:
+			/* Debugger is busy, nothing to do */
+			return;
+		case IANJUTA_DEBUGGER_STOPPED:
 			self->stop_on_sharedlib = FALSE;
-			signal = state;
-		}
-	    break;
-	case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
-		if (self->debugger_state != state) 
-		{
+			signal = DEBUGGER_STOPPED_SIGNAL;
 			self->debugger_state = state;
+			break;
+		case IANJUTA_DEBUGGER_STARTED:
+			self->stop_on_sharedlib = FALSE;
+			signal = self->debugger_state < IANJUTA_DEBUGGER_STARTED ? DEBUGGER_STARTED_SIGNAL : PROGRAM_UNLOADED_SIGNAL;			
+			self->debugger_state = state;
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_LOADED:
+			self->stop_on_sharedlib = FALSE;
+			signal = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_LOADED ? PROGRAM_LOADED_SIGNAL : PROGRAM_EXITED_SIGNAL;			
+			self->debugger_state = state;
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
 			if (!self->stop_on_sharedlib)
 			{
-				signal = state;
+				emit_program_started = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED;
+				signal = PROGRAM_STOPPED_SIGNAL;			
 			}
+			else if (self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED)
+			{
+				signal = PROGRAM_STARTED_SIGNAL;
+			}
+			self->debugger_state = state;
+			break;
+		case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
+			emit_program_started = self->debugger_state < IANJUTA_DEBUGGER_PROGRAM_STOPPED;
+			self->stop_on_sharedlib = FALSE;
+			signal = PROGRAM_RUNNING_SIGNAL;			
+			self->debugger_state = state;
+			break;
 		}
-	    break;
 	}
 
 	self->prepend_command++;
 	/* Emit signal */
 	switch (signal)
 	{
-	case IANJUTA_DEBUGGER_BUSY:
+	case NO_SIGNAL:
 		/* Do nothing */
 		break;
-	case IANJUTA_DEBUGGER_STOPPED:
+	case DEBUGGER_STOPPED_SIGNAL:
+		DEBUG_PRINT("** emit debugger-stopped **");
 		g_signal_emit_by_name (self->plugin, "debugger-stopped", err);
 		break;
-	case IANJUTA_DEBUGGER_STARTED:
+	case DEBUGGER_STARTED_SIGNAL:
+		DEBUG_PRINT("** emit debugger-started **");
 		g_signal_emit_by_name (self->plugin, "debugger-started");
 		break;
-	case IANJUTA_DEBUGGER_PROGRAM_LOADED:
+	case PROGRAM_LOADED_SIGNAL:
+		DEBUG_PRINT("** emit program-loaded **");
 		g_signal_emit_by_name (self->plugin, "program-loaded");
 		break;
-	case IANJUTA_DEBUGGER_PROGRAM_STOPPED:
+	case PROGRAM_UNLOADED_SIGNAL:
+		DEBUG_PRINT("** emit program-unloaded **");
+		g_signal_emit_by_name (self->plugin, "program-unloaded");
+		break;
+	case PROGRAM_STARTED_SIGNAL:
+		DEBUG_PRINT("** emit program-started **");
+		g_signal_emit_by_name (self->plugin, "program-started");
+		break;
+	case PROGRAM_EXITED_SIGNAL:
+		DEBUG_PRINT("** emit program-exited **");
+		g_signal_emit_by_name (self->plugin, "program-exited");
+		break;
+	case PROGRAM_STOPPED_SIGNAL:
+		DEBUG_PRINT("** emit program-exited **");
+		if (emit_program_started)
+		{	
+			DEBUG_PRINT("** emit program-started **");
+			g_signal_emit_by_name (self->plugin, "program-started");
+		}
+		DEBUG_PRINT("** emit program-stopped **");
 		g_signal_emit_by_name (self->plugin, "program-stopped");
 		break;
-	case IANJUTA_DEBUGGER_PROGRAM_RUNNING:
+	case PROGRAM_RUNNING_SIGNAL:
+		if (emit_program_started)
+		{	
+			DEBUG_PRINT("** emit program-started **");
+			g_signal_emit_by_name (self->plugin, "program-started");
+		}
+		DEBUG_PRINT("** emit program-running **");
 		g_signal_emit_by_name (self->plugin, "program-running");
 		break;
 	}
@@ -270,7 +349,6 @@ dma_queue_emit_debugger_ready (DmaDebuggerQueue *self)
 		{
 			anjuta_status_busy_pop (status);
 			self->busy = FALSE;
-			/*g_signal_emit_by_name (self->plugin, "debugger-ready", self->queue_state);*/
 		}
 	}	
 }
@@ -357,6 +435,12 @@ dma_debugger_queue_execute (DmaDebuggerQueue *self)
 			dma_command_free (self->last);
 			self->last = NULL;
 
+			/* Display error message to user */
+			if (err->message != NULL)
+			{
+				anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (self->plugin)->shell), err->message);
+			}
+			
 			g_error_free (err);
 		}
 	}
@@ -463,13 +547,17 @@ dma_debugger_activate_plugin (DmaDebuggerQueue* self, const gchar *mime_type)
 		/* Get debugger interface */
 		self->debugger = (IAnjutaDebugger *)anjuta_plugin_manager_get_plugin_by_id (plugin_manager, value);
 
-		self->feature = 0;
+		self->support = 0;
 		/* Check if cpu interface is available */
-		self->feature |= IANJUTA_IS_CPU_DEBUGGER(self->debugger) ? HAS_CPU : 0;
+		self->support |= IANJUTA_IS_CPU_DEBUGGER(self->debugger) ? HAS_CPU : 0;
 		/* Check if breakpoint interface is available */
-		self->feature |= IANJUTA_IS_BREAKPOINT_DEBUGGER(self->debugger) ? HAS_BREAKPOINT : 0;
+		self->support |= IANJUTA_IS_DEBUGGER_BREAKPOINT(self->debugger) ? HAS_BREAKPOINT : 0;
+		if (IANJUTA_IS_DEBUGGER_BREAKPOINT (self->debugger))
+		{
+			self->support |= ianjuta_debugger_breakpoint_implement (IANJUTA_DEBUGGER_BREAKPOINT (self->debugger), NULL) * HAS_BREAKPOINT * 2;
+		}			
 		/* Check if variable interface is available */
-		self->feature |= IANJUTA_IS_VARIABLE_DEBUGGER(self->debugger) ? HAS_VARIABLE : 0;
+		self->support |= IANJUTA_IS_VARIABLE_DEBUGGER(self->debugger) ? HAS_VARIABLE : 0;
 		
 		g_free (value);
 
@@ -659,7 +747,7 @@ dma_debugger_queue_stop (DmaDebuggerQueue *self)
 		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_dma_frame_changed), self);
 		g_signal_handlers_disconnect_by_func (self->debugger, G_CALLBACK (on_dma_sharedlib_event), self);
 		self->debugger = NULL;
-		self->feature = 0;
+		self->support = 0;
 	}
 }
 
@@ -728,10 +816,10 @@ dma_debugger_queue_get_state (DmaDebuggerQueue *self)
 	return self->queue_state;
 }
 
-gint
-dma_debugger_queue_get_feature (DmaDebuggerQueue *self)
+gboolean
+dma_debugger_queue_is_supported (DmaDebuggerQueue *self, DmaDebuggerCapability capability)
 {
-	return self->feature;
+	return self->support & capability ? TRUE : FALSE;
 }
 
 /* GObject functions
@@ -775,7 +863,7 @@ dma_debugger_queue_instance_init (DmaDebuggerQueue *self)
 {
 	self->plugin = NULL;
 	self->debugger = NULL;
-	self->feature = 0;
+	self->support = 0;
 	self->queue = g_queue_new ();
 	self->last = NULL;
 	self->busy = FALSE;
