@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -31,6 +31,7 @@
 
 #include "utilities.h"
 #include "sharedlib.h"
+#include "queue.h"
 
 static gint
 on_sharedlibs_delete_event (GtkWidget* w, GdkEvent *event, gpointer data)
@@ -88,11 +89,10 @@ on_sharedlibs_update_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
 	Sharedlibs *sl = (Sharedlibs *)user_data;
 	
-	ianjuta_debugger_info_sharedlib (
+	dma_queue_info_sharedlib (
 			sl->debugger,
 			(IAnjutaDebuggerCallback)sharedlibs_update,
-			sl,
-			NULL);
+			sl);
 }
 
 static void
@@ -100,7 +100,7 @@ sharedlibs_update_controls (Sharedlibs* ew)
 {
      gboolean R;
 
-     R = ianjuta_debugger_get_status (ew->debugger, NULL) == IANJUTA_DEBUGGER_OK;
+     R = dma_debugger_queue_get_state (ew->debugger) == IANJUTA_DEBUGGER_OK;
 
      gtk_widget_set_sensitive(ew->widgets.menu_update, R);
 }
@@ -192,7 +192,8 @@ create_sharedlibs_gui (Sharedlibs *sl)
 	gtk_widget_show (label8);
 	gtk_clist_set_column_widget (GTK_CLIST (clist4), 2, label8);
 	
-	label9 = gtk_label_new (_("Sysm Read"));
+	/* Whether the debugging symbols in the shared libraries have been read */
+	label9 = gtk_label_new (_("Symbols read"));
 	gtk_widget_show (label9);
 	gtk_clist_set_column_widget (GTK_CLIST (clist4), 3, label9);
 	
@@ -211,14 +212,13 @@ create_sharedlibs_gui (Sharedlibs *sl)
 }
 
 Sharedlibs*
-sharedlibs_new (IAnjutaDebugger *debugger)
+sharedlibs_new (DebugManagerPlugin *plugin)
 {
 	Sharedlibs* ew;
 	ew = g_malloc(sizeof(Sharedlibs));
 	if(ew)
 	{
-		ew->debugger = debugger;
-		g_object_ref (debugger);
+		ew->debugger = dma_debug_manager_get_queue (plugin);
 		
 		ew->is_showing = FALSE;
 		ew->win_width = 410;
@@ -254,11 +254,10 @@ sharedlibs_show (Sharedlibs* ew)
 										ew->win_width, ew->win_height);
 			gtk_widget_show(ew->widgets.window);
 			ew->is_showing = TRUE;
-			ianjuta_debugger_info_sharedlib (
+			dma_queue_info_sharedlib (
 					ew->debugger,
 					(IAnjutaDebuggerCallback)sharedlibs_update,
-					ew,
-					NULL);
+					ew);
 		}
 	}
 }
@@ -284,7 +283,6 @@ sharedlibs_free(Sharedlibs* sg)
 	if(sg)
 	{
 		sharedlibs_clear(sg);
-		g_object_unref (sg->debugger);
 		gtk_widget_destroy(sg->widgets.window);
 		gtk_widget_destroy(sg->widgets.menu);
 		g_free(sg);
