@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -37,20 +37,21 @@
 
 #include "utilities.h"
 #include "info.h"
+#include "queue.h"
 
 #define ANJUTA_PIXMAP_POINTER PACKAGE_PIXMAPS_DIR"/pointer.png"
 
 typedef struct _DmaThreadStackTrace
 {
 	GtkTreeModel *model;
-	guint thread;
+	gint thread;
 	guint last_update;
 } DmaThreadStackTrace;
 
 struct _StackTrace
 {
 	DebugManagerPlugin *plugin;
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	
 	GtkActionGroup *action_group;
 	
@@ -314,7 +315,7 @@ on_stack_trace_updated (const GList *stack, gpointer user_data)
 }
 
 static DmaThreadStackTrace*
-dma_thread_create_new_stack_trace (StackTrace *self, guint thread)
+dma_thread_create_new_stack_trace (StackTrace *self, gint thread)
 {
 	DmaThreadStackTrace *trace;
 	GtkListStore *store;
@@ -339,11 +340,10 @@ dma_thread_create_new_stack_trace (StackTrace *self, guint thread)
 	self->current = trace;
 		
 	/* Ask debugger to get all frame data */
-	ianjuta_debugger_list_frame (
+	dma_queue_list_frame (
 			self->debugger,
 			(IAnjutaDebuggerCallback)on_stack_trace_updated,
-			self,
-			NULL);
+			self);
 	
 	self->list = g_list_append (self->list, trace);
 	
@@ -353,16 +353,13 @@ dma_thread_create_new_stack_trace (StackTrace *self, guint thread)
 static DmaThreadStackTrace*
 dma_thread_update_stack_trace (StackTrace *self, DmaThreadStackTrace *trace)
 {
-	GtkListStore *store;
-	
 	trace->last_update = self->current_update;
 	
 	/* Ask debugger to get all frame data */
-	ianjuta_debugger_list_frame (
+	dma_queue_list_frame (
 			self->debugger,
 			(IAnjutaDebuggerCallback)on_stack_trace_updated,
-			self,
-			NULL);
+			self);
 	
 	return trace;
 }
@@ -371,13 +368,13 @@ static gboolean
 on_find_stack_trace (gconstpointer a, gconstpointer b)
 {
 	const DmaThreadStackTrace *trace = (const DmaThreadStackTrace *)a;
-	guint thread = (guint)b;
+	guint thread = (gint)b;
 	
 	return trace->thread != thread;
 }
 
 static void
-dma_thread_set_stack_trace (StackTrace *self, guint thread)
+dma_thread_set_stack_trace (StackTrace *self, gint thread)
 {
 	GList *list;
 	DmaThreadStackTrace *trace;
@@ -412,7 +409,7 @@ dma_thread_set_stack_trace (StackTrace *self, guint thread)
  *---------------------------------------------------------------------------*/
 
 static void
-on_frame_changed (StackTrace *self, guint frame, guint thread)
+on_frame_changed (StackTrace *self, guint frame, gint thread)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -468,7 +465,7 @@ on_stack_frame_set_activate (GtkAction *action, gpointer user_data)
 		return;
 	
 	/* issue a command to switch active frame to new location */
-	ianjuta_debugger_set_frame (st->debugger, selected_frame, NULL);
+	dma_queue_set_frame (st->debugger, selected_frame);
 }
 
 static void
@@ -498,7 +495,7 @@ on_stack_view_source_activate (GtkAction *action, gpointer user_data)
 						-1);
 	
 	address = adr != NULL ? strtoul (adr, NULL, 0) : 0;
-	dma_debug_manager_goto_code (st->plugin, uri, line, address);
+	g_signal_emit_by_name (st->plugin, "location-changed", address, uri, line);
 	g_free (uri);
 	g_free (adr);
 }
@@ -642,16 +639,20 @@ create_stack_trace_gui(StackTrace *st)
 	gtk_tree_view_column_set_title (column, _("Function"));
 	gtk_tree_view_append_column (st->treeview, column);
 	
-	column = gtk_tree_view_column_new ();
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_column_pack_start (column, renderer, TRUE);
-	gtk_tree_view_column_add_attribute (column, renderer, "text",
-										STACK_TRACE_ADDR_COLUMN);
-	gtk_tree_view_column_add_attribute (column, renderer, "foreground",
-										STACK_TRACE_COLOR_COLUMN);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_title (column, _("Address"));
-	gtk_tree_view_append_column (st->treeview, column);
+	if (dma_debugger_queue_is_supported (st->debugger, HAS_CPU))
+	{
+		/* Display address only if debugger has such concept */
+		column = gtk_tree_view_column_new ();
+		renderer = gtk_cell_renderer_text_new ();
+		gtk_tree_view_column_pack_start (column, renderer, TRUE);
+		gtk_tree_view_column_add_attribute (column, renderer, "text",
+											STACK_TRACE_ADDR_COLUMN);
+		gtk_tree_view_column_add_attribute (column, renderer, "foreground",
+											STACK_TRACE_COLOR_COLUMN);
+		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+		gtk_tree_view_column_set_title (column, _("Address"));
+		gtk_tree_view_append_column (st->treeview, column);
+	}
 	
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
@@ -702,24 +703,32 @@ destroy_stack_trace_gui (StackTrace *st)
 }
 
 static void
-on_program_stopped (StackTrace *self, guint thread)
+on_program_moved (StackTrace *self, guint pid, gint thread)
 {
 	self->current_update++;
 	dma_thread_set_stack_trace (self, thread);
 }
 
 static void
-on_debugger_started (StackTrace *self)
+on_program_exited (StackTrace *self)
 {
-	self->current_update = 0;
-	create_stack_trace_gui (self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_program_exited), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_program_moved), self);
+	g_signal_handlers_disconnect_by_func (self->plugin, G_CALLBACK (on_frame_changed), self);
+	
+	dma_thread_clear_all_stack_trace (self);
+	destroy_stack_trace_gui (self);
 }
 
 static void
-on_debugger_stopped (StackTrace *self)
+on_program_started (StackTrace *self)
 {
-	dma_thread_clear_all_stack_trace (self);
-	destroy_stack_trace_gui (self);
+	self->current_update = 0;
+	create_stack_trace_gui (self);
+	
+	g_signal_connect_swapped (self->plugin, "program-exited", G_CALLBACK (on_program_exited), self);
+	g_signal_connect_swapped (self->plugin, "program-moved", G_CALLBACK (on_program_moved), self);
+	g_signal_connect_swapped (self->plugin, "frame-changed", G_CALLBACK (on_frame_changed), self);
 }
 
 /* Public functions
@@ -729,7 +738,7 @@ on_debugger_stopped (StackTrace *self)
  *---------------------------------------------------------------------------*/
 
 StackTrace *
-stack_trace_new (IAnjutaDebugger *debugger, DebugManagerPlugin *plugin)
+stack_trace_new (DebugManagerPlugin *plugin)
 {
 	StackTrace *st;
 	AnjutaUI *ui;
@@ -738,8 +747,7 @@ stack_trace_new (IAnjutaDebugger *debugger, DebugManagerPlugin *plugin)
 	if (st == NULL) return NULL;
 
 	st->plugin = plugin;
-	st->debugger = debugger;
-	if (debugger != NULL) g_object_ref (debugger);
+	st->debugger = dma_debug_manager_get_queue (plugin);
 
 	/* Register actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(st->plugin)->shell, NULL);
@@ -750,10 +758,7 @@ stack_trace_new (IAnjutaDebugger *debugger, DebugManagerPlugin *plugin)
 											G_N_ELEMENTS (actions_stack_trace),
 											GETTEXT_PACKAGE, TRUE, st);
 
-	g_signal_connect_swapped (st->debugger, "debugger-started", G_CALLBACK (on_debugger_started), st);
-	g_signal_connect_swapped (st->debugger, "debugger-stopped", G_CALLBACK (on_debugger_stopped), st);
-	g_signal_connect_swapped (st->debugger, "program-stopped", G_CALLBACK (on_program_stopped), st);
-	g_signal_connect_swapped (st->debugger, "frame-changed", G_CALLBACK (on_frame_changed), st);
+	g_signal_connect_swapped (st->plugin, "program-started", G_CALLBACK (on_program_started), st);
 	
 	return st;
 }
@@ -766,14 +771,7 @@ stack_trace_free (StackTrace * st)
 	g_return_if_fail (st != NULL);
 
 	/* Disconnect from debugger */
-	if (st->debugger != NULL)
-	{	
-		g_signal_handlers_disconnect_by_func (st->debugger, G_CALLBACK (on_debugger_started), st);
-		g_signal_handlers_disconnect_by_func (st->debugger, G_CALLBACK (on_debugger_stopped), st);
-		g_signal_handlers_disconnect_by_func (st->debugger, G_CALLBACK (on_program_stopped), st);
-		g_signal_handlers_disconnect_by_func (st->debugger, G_CALLBACK (on_frame_changed), st);
-		g_object_unref (st->debugger);	
-	}
+	g_signal_handlers_disconnect_matched (st->plugin, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, st);	
 
 	/* Remove menu actions */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (st->plugin)->shell, NULL);
