@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <libanjuta/resources.h>
@@ -51,7 +51,7 @@ static gpointer parent_class;
 struct _AnjutaDocmanPriv {
 	DocmanPlugin *plugin;
 	AnjutaPreferences *preferences;
-	IAnjutaEditor *current_editor;
+	IAnjutaDocument *current_editor;
 	
 	GtkWidget *fileselection;
 	
@@ -73,10 +73,6 @@ struct _AnjutaDocmanPage {
 
 typedef struct _AnjutaDocmanPage AnjutaDocmanPage;
 
-static void on_notebook_switch_page (GtkNotebook * notebook,
-									 GtkNotebookPage * page,
-									 gint page_num, AnjutaDocman *docman);
-
 
 static void anjuta_docman_order_tabs(AnjutaDocman *docman);
 static void anjuta_docman_update_page_label (AnjutaDocman *docman, GtkWidget *te);
@@ -95,12 +91,12 @@ on_text_editor_notebook_close_page (GtkButton* button,
 	while (node)
 	{
 		AnjutaDocmanPage *page;
-		IAnjutaEditor *te;
+		IAnjutaDocument *doc;
 		page = (AnjutaDocmanPage *) node->data;
 		if (page->close_button == GTK_WIDGET(button))
 		{
-			te = IANJUTA_EDITOR (page->widget);
-			anjuta_docman_set_current_editor(docman, te);
+			doc = IANJUTA_DOCUMENT (page->widget);
+			anjuta_docman_set_current_document(docman, doc);
 			break;
 		}
 		node = g_list_next (node);
@@ -185,12 +181,12 @@ editor_tab_widget_new(AnjutaDocmanPage* page, AnjutaDocman* docman)
 						  _("Close file"),
 						  NULL);
 
-	label = gtk_label_new (ianjuta_editor_get_filename (IANJUTA_EDITOR(page->widget),
+	label = gtk_label_new (ianjuta_document_get_filename (IANJUTA_DOCUMENT(page->widget),
 														NULL));
 	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
 	gtk_widget_show (label);
 
-	menu_label = gtk_label_new (ianjuta_editor_get_filename (IANJUTA_EDITOR(page->widget),
+	menu_label = gtk_label_new (ianjuta_document_get_filename (IANJUTA_DOCUMENT(page->widget),
 															NULL));
 	gtk_widget_show (menu_label);
   	 
@@ -357,7 +353,7 @@ anjuta_docman_open_file (AnjutaDocman *docman)
 }
 
 gboolean
-anjuta_docman_save_editor_as (AnjutaDocman *docman, IAnjutaEditor *te,
+anjuta_docman_save_document_as (AnjutaDocman *docman, IAnjutaDocument *te,
 							  GtkWidget *parent_window)
 {
 	gchar* uri;
@@ -370,7 +366,7 @@ anjuta_docman_save_editor_as (AnjutaDocman *docman, IAnjutaEditor *te,
 	gboolean file_saved = TRUE;
 	
 	g_return_val_if_fail (ANJUTA_IS_DOCMAN (docman), FALSE);
-	g_return_val_if_fail (IANJUTA_IS_EDITOR (te), FALSE);
+	g_return_val_if_fail (IANJUTA_IS_DOCUMENT (te), FALSE);
 	
 	if (parent_window)
 		parent = parent_window;
@@ -384,7 +380,7 @@ anjuta_docman_save_editor_as (AnjutaDocman *docman, IAnjutaEditor *te,
 		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER(dialog), file_uri);
 		g_free (file_uri);
 	}
-	else if ((filename = ianjuta_editor_get_filename(te, NULL)) != NULL)
+	else if ((filename = ianjuta_document_get_filename(IANJUTA_DOCUMENT(te), NULL)) != NULL)
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), filename);
 	else
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), "");
@@ -439,7 +435,7 @@ anjuta_docman_save_editor_as (AnjutaDocman *docman, IAnjutaEditor *te,
 }
 
 gboolean
-anjuta_docman_save_editor (AnjutaDocman *docman, IAnjutaEditor *te,
+anjuta_docman_save_document (AnjutaDocman *docman, IAnjutaDocument *te,
 						   GtkWidget *parent_window)
 {
 	gchar *uri;
@@ -449,8 +445,8 @@ anjuta_docman_save_editor (AnjutaDocman *docman, IAnjutaEditor *te,
 	
 	if (uri == NULL)
 	{
-		anjuta_docman_set_current_editor (docman, te);
-		ret = anjuta_docman_save_editor_as (docman, te, parent_window);
+		anjuta_docman_set_current_document (docman, IANJUTA_DOCUMENT(te));
+		ret = anjuta_docman_save_document_as (docman, te, parent_window);
 	}
 	else
 	{
@@ -488,7 +484,7 @@ anjuta_docman_dispose (GObject *obj)
 		node = editors;
 		while (node)
 		{
-			anjuta_docman_remove_editor (docman, IANJUTA_EDITOR (node->data));
+			anjuta_docman_remove_document (docman, IANJUTA_DOCUMENT (node->data));
 			node = g_list_next (node);
 		}
 		g_list_free (docman->priv->editors);
@@ -596,7 +592,7 @@ on_notebook_switch_page (GtkNotebook * notebook,
 		 * at the beginning of the tab list
 		 */
 		widget = gtk_notebook_get_nth_page (notebook, page_num);
-		anjuta_docman_set_current_editor (docman, IANJUTA_EDITOR (widget));
+		anjuta_docman_set_current_document (docman, IANJUTA_DOCUMENT (widget));
 		
 		if (!g_tabbing &&
 			!anjuta_preferences_get_int (docman->priv->preferences, EDITOR_TABS_ORDERING) && 
@@ -608,7 +604,7 @@ on_notebook_switch_page (GtkNotebook * notebook,
 }
 
 static AnjutaDocmanPage *
-anjuta_docman_page_from_widget (AnjutaDocman *docman, IAnjutaEditor *te)
+anjuta_docman_page_from_widget (AnjutaDocman *docman, IAnjutaDocument *te)
 {
 	GList *node;
 	node = docman->priv->editors;
@@ -625,14 +621,14 @@ anjuta_docman_page_from_widget (AnjutaDocman *docman, IAnjutaEditor *te)
 }
 
 static void
-on_editor_save_point (IAnjutaEditor *editor, gboolean entering,
+on_editor_save_point (IAnjutaDocument *editor, gboolean entering,
 					  AnjutaDocman *docman)
 {
 	anjuta_docman_update_page_label (docman, GTK_WIDGET (editor));
 }
 
 static void
-on_editor_destroy (IAnjutaEditor *te, AnjutaDocman *docman)
+on_editor_destroy (IAnjutaDocument *te, AnjutaDocman *docman)
 {
 	// GtkWidget *submenu;
 	// GtkWidget *wid;
@@ -663,17 +659,17 @@ on_editor_destroy (IAnjutaEditor *te, AnjutaDocman *docman)
 	if (docman->priv->shutingdown == FALSE)
 	{
 		if (docman->priv->current_editor == te)
-			anjuta_docman_set_current_editor (docman, NULL);
+			anjuta_docman_set_current_document(docman, NULL);
 	
 		if (GTK_NOTEBOOK (docman)->children == NULL)
-			anjuta_docman_set_current_editor (docman, NULL);
+			anjuta_docman_set_current_document (docman, NULL);
 		else
 		{
 			GtkWidget *current_editor;
 			page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (docman));
 			current_editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (docman),
 														page_num);
-			anjuta_docman_set_current_editor (docman, IANJUTA_EDITOR (current_editor));
+			anjuta_docman_set_current_document (docman, IANJUTA_DOCUMENT(current_editor));
 		}
 	}
 	g_object_unref (G_OBJECT(te));
@@ -688,12 +684,7 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 {
 	IAnjutaEditor *te;
 	IAnjutaEditorFactory* factory;
-	GtkWidget *event_box;
-	gchar *tip;
-	gchar *ruri;
-
-	EditorTooltips *tooltips = NULL;
-	AnjutaDocmanPage *page;
+	
 	AnjutaStatus *status;
 
 	status = anjuta_shell_get_status (docman->shell, NULL);
@@ -721,8 +712,23 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 	}
 	ianjuta_editor_set_popup_menu (te, docman->priv->popup_menu, NULL);
 	
+	anjuta_docman_add_document(docman, IANJUTA_DOCUMENT(te), uri);
 	
+	return te;
+}
+
+void
+anjuta_docman_add_document (AnjutaDocman *docman, IAnjutaDocument* te, 
+							const gchar* uri)
+{
+	AnjutaDocmanPage *page;
+	GtkWidget *event_box;
+	gchar *tip;
+	gchar *ruri;
+	EditorTooltips *tooltips = NULL;
+
 	gtk_widget_show (GTK_WIDGET(te));
+	
 	page = anjuta_docman_page_new (GTK_WIDGET(te), docman);
 
 	if (tooltips == NULL) {
@@ -730,7 +736,12 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 	}
 
 	docman->priv->editors = g_list_append (docman->priv->editors, (gpointer)page);
-
+	
+	if (!uri)
+	{
+		uri  = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
+	}
+	
 	ruri =  gnome_vfs_format_uri_for_display (uri ? uri : "");
 	
 	/* set the tooltips */	
@@ -770,17 +781,17 @@ anjuta_docman_add_editor (AnjutaDocman *docman, const gchar *uri,
 					  G_CALLBACK (on_editor_destroy), docman);
 				   
 	g_signal_emit_by_name (docman, "editor-added", te);
-	anjuta_docman_set_current_editor(docman, te);
-	anjuta_shell_present_widget( ANJUTA_SHELL (docman->shell), GTK_WIDGET (docman), NULL);
-	g_object_ref(G_OBJECT(te));
-	return te;
+	anjuta_docman_set_current_document(docman, IANJUTA_DOCUMENT(te));
+	anjuta_shell_present_widget( ANJUTA_SHELL (docman->shell), 
+								GTK_WIDGET (docman->priv->plugin->vbox), NULL);
+	g_object_ref(G_OBJECT(te));	
 }
 
 void
-anjuta_docman_remove_editor (AnjutaDocman *docman, IAnjutaEditor* te)
+anjuta_docman_remove_document (AnjutaDocman *docman, IAnjutaDocument* te)
 {
 	if (!te)
-		te = anjuta_docman_get_current_editor (docman);
+		te = anjuta_docman_get_current_document (docman);
 
 	if (te == NULL)
 		return;
@@ -798,19 +809,20 @@ anjuta_docman_set_popup_menu (AnjutaDocman *docman, GtkWidget *menu)
 	docman->priv->popup_menu = menu;
 }
 
-IAnjutaEditor *
-anjuta_docman_get_current_editor (AnjutaDocman *docman)
+IAnjutaDocument *
+anjuta_docman_get_current_document (AnjutaDocman *docman)
 {
 	return docman->priv->current_editor;
 }
 
 void
-anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
+anjuta_docman_set_current_document(AnjutaDocman *docman, IAnjutaDocument * doc)
 {
 	gchar *uri;
-	IAnjutaEditor *ote = docman->priv->current_editor;
+	IAnjutaDocument *ote = docman->priv->current_editor;
+	IAnjutaEditor* te = NULL;
 	
-	if (ote == te)
+	if (ote == doc)
 		return;
 	
 	if (ote != NULL)
@@ -824,11 +836,11 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 			page->is_current = FALSE;
 		}
 	}
-	docman->priv->current_editor = te;
-	if (te != NULL)
+	docman->priv->current_editor = doc;
+	if (doc != NULL)
 	{
 		gint page_num;
-		AnjutaDocmanPage *page = anjuta_docman_page_from_widget (docman, te);
+		AnjutaDocmanPage *page = anjuta_docman_page_from_widget (docman, doc);
 		if (page && page->close_button != NULL)
 		{
 			gtk_widget_set_sensitive (page->close_image, TRUE);
@@ -837,7 +849,7 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 			page->is_current = TRUE;
 		}
 		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (docman),
-										  GTK_WIDGET (te));
+										  GTK_WIDGET (doc));
 		g_signal_handlers_block_by_func (GTK_OBJECT (docman),
 						   GTK_SIGNAL_FUNC (on_notebook_switch_page),
 						   docman);
@@ -847,7 +859,7 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 										EDITOR_TABS_ORDERING))
 				anjuta_docman_order_tabs (docman);
 		
-		gtk_widget_grab_focus (GTK_WIDGET (te));
+		gtk_widget_grab_focus (GTK_WIDGET (doc));
 		anjuta_docman_grab_text_focus (docman);
 		
 		gtk_signal_handler_unblock_by_func (GTK_OBJECT (docman),
@@ -861,26 +873,33 @@ anjuta_docman_set_current_editor (AnjutaDocman *docman, IAnjutaEditor * te)
 		chdir (dir);
 	}
 */
-	uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
-	if (te && uri)
+	uri = NULL;
+	if (IANJUTA_IS_EDITOR(doc))
 	{
-		gchar *hostname;
-		gchar *filename;
-		
-		filename = g_filename_from_uri (uri, &hostname, NULL);
-		if (hostname == NULL && filename)
+		te = IANJUTA_EDITOR(doc);
+
+		if (te)
+			uri = ianjuta_file_get_uri (IANJUTA_FILE(te), NULL);
+		if (te && uri)
 		{
-			gchar *dir;
-			dir = g_path_get_dirname (filename);
-			if (dir)
-				chdir (dir);
-			g_free (dir);
+			gchar *hostname;
+			gchar *filename;
+			
+			filename = g_filename_from_uri (uri, &hostname, NULL);
+			if (hostname == NULL && filename)
+			{
+				gchar *dir;
+				dir = g_path_get_dirname (filename);
+				if (dir)
+					chdir (dir);
+				g_free (dir);
+			}
+			g_free (hostname);
+			g_free (filename);
 		}
-		g_free (hostname);
-		g_free (filename);
+		g_free (uri);
 	}
-	g_free (uri);
-	g_signal_emit_by_name (G_OBJECT (docman), "editor_changed", te);
+	g_signal_emit_by_name (G_OBJECT (docman), "editor_changed", doc);
 	return;
 }
 
@@ -902,6 +921,7 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 	gboolean is_local_uri;
 	gchar *normalized_path = NULL;
 	
+	IAnjutaDocument *doc;
 	IAnjutaEditor *te;
 
 	g_return_val_if_fail (fname, NULL);
@@ -943,8 +963,8 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 		gchar *te_normalized_path = NULL;
 		
 		page = (AnjutaDocmanPage *) node->data;
-		te = IANJUTA_EDITOR (page->widget);
-		te_uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
+		doc = IANJUTA_DOCUMENT (page->widget);
+		te_uri = ianjuta_file_get_uri(IANJUTA_FILE(doc), NULL);
 		
 		if (te_uri == NULL)
 		{
@@ -962,27 +982,31 @@ anjuta_docman_goto_file_line_mark (AnjutaDocman *docman, const gchar *fname,
 			te_normalized_path = g_strdup (te_uri);
 		gnome_vfs_uri_unref (vfs_uri);
 		
-		if (strcmp (normalized_path, te_normalized_path) == 0)
+		if (strcmp (normalized_path, te_normalized_path) == 0
+			&& IANJUTA_IS_EDITOR(doc))
 		{
-			if (lineno >= 0)
+			if (lineno >= 0 && IANJUTA_IS_EDITOR(doc))
 			{
-				ianjuta_editor_goto_line(te, lineno, NULL);
+				ianjuta_editor_goto_line(IANJUTA_EDITOR(doc), lineno, NULL);
 				if (mark)
   	            {
-  	            	ianjuta_markable_delete_all_markers(IANJUTA_MARKABLE(te),
+  	            	ianjuta_markable_delete_all_markers(IANJUTA_MARKABLE(doc),
   	                									IANJUTA_MARKABLE_LINEMARKER,
   	                                                    NULL);
-					ianjuta_markable_mark(IANJUTA_MARKABLE(te), lineno,
+					ianjuta_markable_mark(IANJUTA_MARKABLE(doc), lineno,
   	                				IANJUTA_MARKABLE_LINEMARKER, NULL);
 				}
   	        }
-			anjuta_docman_show_editor (docman, GTK_WIDGET (te));
+			anjuta_docman_show_editor (docman, GTK_WIDGET (doc));
 			an_file_history_push (te_uri, lineno);
 			g_free (uri);
 			g_free (te_uri);
 			g_free (normalized_path);
 			g_free (te_normalized_path);
-			return te;
+			if (IANJUTA_IS_EDITOR(doc))
+				return IANJUTA_EDITOR(doc);
+			else
+				return NULL;
 		}
 		g_free (te_uri);
 		g_free (te_normalized_path);
@@ -1025,7 +1049,7 @@ get_real_path(const gchar *file_name)
 gchar *
 anjuta_docman_get_full_filename (AnjutaDocman *docman, const gchar *fn)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	GList *te_list;
 	gchar *real_path;
 	gchar *fname;
@@ -1049,9 +1073,9 @@ anjuta_docman_get_full_filename (AnjutaDocman *docman, const gchar *fn)
 	fname = g_path_get_basename (fn);
 	
 	/* Next, check if the current text editor buffer matches this name */
-	if (NULL != (te = anjuta_docman_get_current_editor(docman)))
+	if (NULL != (te = anjuta_docman_get_current_document(docman)))
 	{
-		if (strcmp(ianjuta_editor_get_filename(te, NULL), fname) == 0)
+		if (strcmp(ianjuta_document_get_filename(te, NULL), fname) == 0)
 		{
 			g_free (fname);
 			return ianjuta_file_get_uri (IANJUTA_FILE(te), NULL);
@@ -1062,8 +1086,8 @@ anjuta_docman_get_full_filename (AnjutaDocman *docman, const gchar *fn)
 	{
 		AnjutaDocmanPage *page;
 		page = (AnjutaDocmanPage *) te_list->data;
-		te = IANJUTA_EDITOR (page->widget);
-		if (strcmp(fname, ianjuta_editor_get_filename(te, NULL)) == 0)
+		te = IANJUTA_DOCUMENT (page->widget);
+		if (strcmp(fname, ianjuta_document_get_filename(te, NULL)) == 0)
 		{
 			g_free (fname);
 			return ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
@@ -1091,8 +1115,7 @@ anjuta_docman_show_editor (AnjutaDocman *docman, GtkWidget* te)
 		if (t && t == te)
 		{
 			gtk_notebook_set_current_page (GTK_NOTEBOOK (docman), i);
-			anjuta_docman_set_current_editor (docman, IANJUTA_EDITOR (te));
-			anjuta_shell_present_widget (ANJUTA_SHELL (docman->shell), GTK_WIDGET (docman), NULL);
+			anjuta_docman_set_current_document (docman, IANJUTA_DOCUMENT (te));
 			return;
 		}
 		i++;
@@ -1103,14 +1126,15 @@ anjuta_docman_show_editor (AnjutaDocman *docman, GtkWidget* te)
 static void
 anjuta_docman_update_page_label (AnjutaDocman *docman, GtkWidget *te_widget)
 {
-	GdkColor tmpcolor, *colorp = NULL;
 	AnjutaDocmanPage *page;
 	gchar *basename;
 	gchar *uri;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	const gchar* te_filename;
+	gchar* dirty_char;
+	gchar* label;
 	
-	te = IANJUTA_EDITOR (te_widget);
+	te = IANJUTA_DOCUMENT (te_widget);
 	if (te == NULL)
 		return;
 
@@ -1120,47 +1144,38 @@ anjuta_docman_update_page_label (AnjutaDocman *docman, GtkWidget *te_widget)
 	
 	if (!ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE(te), NULL))
 	{
-		/* setting GdkColor value in gtk_widget_modify_fg to NULL will 
-		 * reset it to the default colors */
-		colorp = NULL;
+		dirty_char = NULL;
 	}
 	else
 	{
-		gdk_color_parse("red", &tmpcolor);
-		colorp = &tmpcolor;
+		dirty_char = "*";
 	}
-	gtk_widget_modify_fg (page->label, GTK_STATE_NORMAL, colorp);
-	gtk_widget_modify_fg (page->label, GTK_STATE_INSENSITIVE, colorp);
-	gtk_widget_modify_fg (page->label, GTK_STATE_ACTIVE, colorp);
-	gtk_widget_modify_fg (page->label, GTK_STATE_PRELIGHT, colorp);
-	gtk_widget_modify_fg (page->label, GTK_STATE_SELECTED, colorp);
-
-    gtk_widget_modify_fg (page->menu_label, GTK_STATE_NORMAL, colorp);
-	gtk_widget_modify_fg (page->menu_label, GTK_STATE_INSENSITIVE, colorp);
-  	gtk_widget_modify_fg (page->menu_label, GTK_STATE_ACTIVE, colorp);
-  	gtk_widget_modify_fg (page->menu_label, GTK_STATE_PRELIGHT, colorp);
-  	gtk_widget_modify_fg (page->menu_label, GTK_STATE_SELECTED, colorp);
 	
 	uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
 	if (uri)
 	{
 		basename = g_path_get_basename (uri);
-		gtk_label_set_text (GTK_LABEL (page->label), basename);
-		gtk_label_set_text (GTK_LABEL (page->menu_label), basename);
+		label = g_strconcat(basename, dirty_char, NULL);
+		gtk_label_set_text (GTK_LABEL (page->label), label);
+		gtk_label_set_text (GTK_LABEL (page->menu_label), label);
+		g_free (label);
 		g_free (basename);
 		g_free (uri);
 	}
-	else if ((te_filename = ianjuta_editor_get_filename(te, NULL)) != NULL)
+	else if ((te_filename = ianjuta_document_get_filename(te, NULL)) != NULL)
 	{
-		gtk_label_set_text (GTK_LABEL (page->label), te_filename);
-		gtk_label_set_text (GTK_LABEL (page->menu_label), te_filename);	
+		label = g_strconcat(te_filename, dirty_char, NULL);
+		gtk_label_set_text (GTK_LABEL (page->label), label);
+		gtk_label_set_text (GTK_LABEL (page->menu_label), label);
+		g_free (label);
 	}
 }
 
 static void
 anjuta_docman_grab_text_focus (AnjutaDocman *docman)
 {
-	anjuta_shell_present_widget(docman->shell, GTK_WIDGET(docman), NULL);
+	anjuta_shell_present_widget(docman->shell, 
+								GTK_WIDGET(docman->priv->plugin->vbox), NULL);
 }
 
 void
@@ -1206,12 +1221,12 @@ anjuta_docman_delete_all_indicators (AnjutaDocman *docman)
 	}
 }
 
-IAnjutaEditor *
-anjuta_docman_get_editor_from_path (AnjutaDocman *docman, const gchar *szFullPath)
+IAnjutaDocument *
+anjuta_docman_get_document_from_path (AnjutaDocman *docman, const gchar *szFullPath)
 {
 	GList *node;
 	AnjutaDocmanPage *page;
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 	
 	g_return_val_if_fail (szFullPath != NULL, NULL);
 	node = docman->priv->editors;
@@ -1219,7 +1234,7 @@ anjuta_docman_get_editor_from_path (AnjutaDocman *docman, const gchar *szFullPat
 	{
 		gchar* uri;
 		page = node->data;
-		te = IANJUTA_EDITOR (page->widget);
+		te = IANJUTA_DOCUMENT (page->widget);
 		uri = ianjuta_file_get_uri (IANJUTA_FILE(te), NULL);
 		if (uri != NULL)
 		{
@@ -1239,11 +1254,11 @@ anjuta_docman_get_editor_from_path (AnjutaDocman *docman, const gchar *szFullPat
 void 
 anjuta_docman_save_file_if_modified (AnjutaDocman *docman, const gchar *szFullPath)
 {
-	IAnjutaEditor *te;
+	IAnjutaDocument *te;
 
 	g_return_if_fail ( szFullPath != NULL );
 
-	te = anjuta_docman_get_editor_from_path (docman, szFullPath);
+	te = anjuta_docman_get_document_from_path (docman, szFullPath);
 	if( NULL != te )
 	{
 		if(!ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE(te), NULL))
@@ -1262,7 +1277,7 @@ anjuta_docman_reload_file (AnjutaDocman *docman, const gchar *szFullPath)
 
 	g_return_if_fail (szFullPath != NULL);
 
-	te = anjuta_docman_get_editor_from_path (docman, szFullPath);
+	te = IANJUTA_EDITOR(anjuta_docman_get_document_from_path (docman, szFullPath));
 	if( NULL != te )
 	{
 		glong nNowPos =  ianjuta_editor_get_lineno(te, NULL);
@@ -1310,7 +1325,7 @@ anjuta_docman_order_tabs (AnjutaDocman *docman)
 		else
 		{
 			tab_labels[i].m_widget = widget;
-			tab_labels[i].m_label = ianjuta_editor_get_filename (IANJUTA_EDITOR(widget),
+			tab_labels[i].m_label = ianjuta_document_get_filename (IANJUTA_DOCUMENT(widget),
 																NULL);
 		}
 	}
@@ -1361,7 +1376,7 @@ anjuta_docman_find_editor_with_path (AnjutaDocman *docman,
 		page = (AnjutaDocmanPage *) tmp->data;
 		if (!page)
 			continue;
-		te = IANJUTA_EDITOR(page->widget);
+		te = IANJUTA_IS_EDITOR(page->widget) ? IANJUTA_EDITOR(page->widget) : NULL;
 		if (!te)
 			continue;
 		uri = ianjuta_file_get_uri (IANJUTA_FILE(te), NULL);
@@ -1386,10 +1401,8 @@ anjuta_docman_get_all_editors (AnjutaDocman *docman)
 	while (node)
 	{
 		AnjutaDocmanPage *page;
-		IAnjutaEditor *te;
 		page = (AnjutaDocmanPage *) node->data;
-		te = IANJUTA_EDITOR (page->widget);
-		editors = g_list_prepend (editors, te);
+		editors = g_list_prepend (editors, page->widget);
 		node = g_list_next (node);
 	}
 	editors = g_list_reverse (editors);
@@ -1399,20 +1412,7 @@ anjuta_docman_get_all_editors (AnjutaDocman *docman)
 void
 anjuta_docman_set_busy (AnjutaDocman *docman, gboolean state)
 {
-	GList *node;
-	
-	node = docman->priv->editors;
-	while (node)
-	{
-		AnjutaDocmanPage *page;
-		IAnjutaEditor *te;
-		page = (AnjutaDocmanPage *) node->data;
-		te = IANJUTA_EDITOR (page->widget);
-		/* FIXME:
-		text_editor_set_busy (te, state);*/
-		node = g_list_next (node);
-	}
-	gdk_flush ();
+	/* FIXME: Maybe we do not longer need this */
 }
 
 ANJUTA_TYPE_BEGIN(AnjutaDocman, anjuta_docman, GTK_TYPE_NOTEBOOK);
