@@ -52,7 +52,7 @@
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-document-manager.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-document-manager.glade"
-#define ICON_FILE "anjuta-document-manager.png"
+#define ICON_FILE "anjuta-document-manager-plugin-48.png"
 
 /* Pixmaps */
 #define ANJUTA_PIXMAP_SWAP                "anjuta-swap"
@@ -156,7 +156,7 @@ static GtkActionEntry actions_file[] = {
     G_CALLBACK (on_new_file_activate)},
   { "ActionFileSave", GTK_STOCK_SAVE, N_("_Save"), "<control>s",
 	N_("Save current file"), G_CALLBACK (on_save_activate)},
-  { "ActionFileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), NULL,
+  { "ActionFileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."),  "<shift><control>s",
 	N_("Save the current file with a different name"),
     G_CALLBACK (on_save_as_activate)},
   { "ActionFileSaveAll", NULL, N_("Save A_ll"), NULL,
@@ -605,8 +605,10 @@ update_editor_ui_disable_all (AnjutaPlugin *plugin)
 		{
 			action = anjuta_ui_get_action (ui, action_groups[i].name,
 										   action_groups[i].group[j].name);
-			if (action_groups[i].group[j].callback)
+			if (action_groups[i].group[j].callback &&
+				strcmp (action_groups[i].group[j].name, "ActionFileNew") != 0)
 			{
+				/* Disable all but ActoinFileNew actions */
 				g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 			}
 		}
@@ -659,12 +661,13 @@ update_editor_ui_interface_items (AnjutaPlugin *plugin, IAnjutaDocument *editor)
 	/* Check if it is a C or C++ file */
 	if (language && flag)
 	{
-
-		
 		const gchar* lang_name =
-			ianjuta_language_get_name_from_editor (language, IANJUTA_EDITOR_LANGUAGE (editor), NULL);
+			ianjuta_language_get_name_from_editor (language,
+												   IANJUTA_EDITOR_LANGUAGE (editor),
+												   NULL);
 		
-		if (lang_name && (g_str_equal (lang_name, "C") || g_str_equal (lang_name, "C++")))
+		if (lang_name && (g_str_equal (lang_name, "C") ||
+						  g_str_equal (lang_name, "C++")))
 		{
 			flag = TRUE;
 		}
@@ -1008,14 +1011,18 @@ create_highlight_submenu (DocmanPlugin *plugin, IAnjutaEditor *editor)
 		const gchar *lang = node->data;
 		const gchar *name = ianjuta_editor_language_get_language_name (IANJUTA_EDITOR_LANGUAGE (editor), lang, NULL);
 		
-		menuitem = gtk_menu_item_new_with_mnemonic (name);
-		g_object_set_data_full (G_OBJECT (menuitem), "language_code",
-								g_strdup (lang),
-								(GDestroyNotify)g_free);
-		g_signal_connect (G_OBJECT (menuitem), "activate",
-						  G_CALLBACK (on_force_hilite_activate),
-						  plugin);
-		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+		/* Should fix #493583 */
+		if (name != NULL)
+		{
+			menuitem = gtk_menu_item_new_with_mnemonic (name);
+			g_object_set_data_full (G_OBJECT (menuitem), "language_code",
+									g_strdup (lang),
+									(GDestroyNotify)g_free);
+			g_signal_connect (G_OBJECT (menuitem), "activate",
+							  G_CALLBACK (on_force_hilite_activate),
+							  plugin);
+			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menuitem);
+		}
 		node = g_list_next (node);
 	}
 	g_list_free(sorted_languages);
@@ -1173,13 +1180,9 @@ on_editor_changed (AnjutaDocman *docman, IAnjutaDocument *te,
 			for (node = support_plugin_descs; node != NULL; node = g_list_next (node))
 			{
 				gchar *plugin_id;
-				gchar *languages;
 				
 				AnjutaPluginDescription *desc = node->data;
 				
-				anjuta_plugin_description_get_string (desc, "Language Support",
-													  "Languages", &languages);
-				
 				anjuta_plugin_description_get_string (desc, "Anjuta Plugin", "Location",
 													  &plugin_id);
 				
