@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -38,13 +38,14 @@
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-editor-convert.h>
 #include <libanjuta/interfaces/ianjuta-editor-line-mode.h>
-#include <libanjuta/interfaces/ianjuta-editor-autocomplete.h>
 #include <libanjuta/interfaces/ianjuta-editor-view.h>
 #include <libanjuta/interfaces/ianjuta-editor-folds.h>
 #include <libanjuta/interfaces/ianjuta-editor-comment.h>
 #include <libanjuta/interfaces/ianjuta-editor-zoom.h>
 #include <libanjuta/interfaces/ianjuta-editor-goto.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
+#include <libanjuta/interfaces/ianjuta-editor-assist.h>
+#include <libanjuta/interfaces/ianjuta-editor-search.h>
 #include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-editor-factory.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -52,6 +53,7 @@
 #include <libanjuta/interfaces/ianjuta-markable.h>
 #include <libanjuta/interfaces/ianjuta-indicable.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
+#include <libanjuta/interfaces/ianjuta-document.h>
 
 #include "properties.h"
 #include "text_editor.h"
@@ -79,19 +81,19 @@
 #define TEXT_EDITOR_LINEMARKER              4
 
 /* Include marker pixmaps */
-#include "bookmark.xpm"
-#include "breakpoint-disabled.xpm"
-#include "breakpoint-enabled.xpm"
-#include "program-counter.xpm"
-#include "linemarker.xpm"
+#include "anjuta-bookmark-16.xpm"
+#include "anjuta-breakpoint-disabled-16.xpm"
+#include "anjuta-breakpoint-enabled-16.xpm"
+#include "anjuta-pcmark-16.xpm"
+#include "anjuta-linemark-16.xpm"
 
 static gchar** marker_pixmap[] = 
 {
-	bookmark_xpm,
-	breakpoint_disabled_xpm,
-	breakpoint_enabled_xpm,
-	program_counter_xpm,
-	linemarker_xpm,
+	anjuta_bookmark_16_xpm,
+	anjuta_breakpoint_disabled_16_xpm,
+	anjuta_breakpoint_enabled_16_xpm,
+	anjuta_pcmark_16_xpm,
+	anjuta_linemark_16_xpm,
 	NULL
 };
 
@@ -924,7 +926,8 @@ text_editor_set_indicator (TextEditor *te, gint start,
 	g_return_val_if_fail (IS_SCINTILLA (te->scintilla) == TRUE, -1);
 
 	if (start >= 0) {
-		if (end <= start)
+		end --;	/* supplied end-location is one-past the last char to process */
+		if (end < start)
 			return -1;
 
 		do
@@ -1855,12 +1858,26 @@ text_editor_get_num_bookmarks(TextEditor* te)
 	return nMarkers ;
 }
 
+/* Gets the word before just before carat */
+gchar*
+text_editor_get_word_before_carat (TextEditor *te)
+{
+	gchar buffer[512];
+	buffer[0] = '\0';
+	aneditor_command (TEXT_EDITOR (te)->editor_id,
+					  ANE_GETWORDBEFORECARAT, (glong) buffer, 512);
+	if (buffer[0] != '\0')
+		return g_strdup (buffer);
+	else
+		return NULL;
+}
+
 /*
  *Get the current selection. If there is no selection, or if the selection
  * is all blanks, get the word under teh cursor.
  */
-gchar
-*text_editor_get_current_word(TextEditor *te)
+gchar*
+text_editor_get_current_word (TextEditor *te)
 {
 	char *buf = text_editor_get_selection(te);
 	if (buf)
@@ -2190,6 +2207,14 @@ itext_editor_get_position (IAnjutaEditor *editor, GError **e)
 	return text_editor_get_current_position (TEXT_EDITOR(editor));
 }
 
+static IAnjutaIterable*
+itext_editor_get_position_iter (IAnjutaEditor* editor, GError **e)
+{
+	TextEditor* te = TEXT_EDITOR (editor);
+	return IANJUTA_ITERABLE (text_editor_cell_new (te,
+												   text_editor_get_current_position (te)));
+}
+
 static gint
 itext_editor_get_lineno (IAnjutaEditor *editor, GError **e)
 {
@@ -2275,50 +2300,6 @@ itext_editor_erase_all (IAnjutaEditor *editor, GError **e)
 							0, 0);
 }
 
-static const gchar *
-itext_editor_get_filename (IAnjutaEditor *editor, GError **e)
-{
-	return (TEXT_EDITOR (editor))->filename;
-}
-
-static gboolean
-itext_editor_can_undo(IAnjutaEditor *editor, GError **e)
-{
-	return text_editor_can_undo(TEXT_EDITOR(editor));
-}
-
-static gboolean
-itext_editor_can_redo(IAnjutaEditor *editor, GError **e)
-{
-	return text_editor_can_redo(TEXT_EDITOR(editor));
-}
-
-static void 
-itext_editor_undo(IAnjutaEditor* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_UNDO, 0, 0);
-}
-
-static void 
-itext_editor_begin_undo_action (IAnjutaEditor* te, GError** ee)
-{
-	scintilla_send_message (SCINTILLA (TEXT_EDITOR (te)->scintilla),
-							SCI_BEGINUNDOACTION, 0, 0);
-}
-
-static void 
-itext_editor_end_undo_action (IAnjutaEditor* te, GError** ee)
-{
-	scintilla_send_message (SCINTILLA (TEXT_EDITOR (te)->scintilla),
-							SCI_ENDUNDOACTION, 0, 0);
-}
-
-static void 
-itext_editor_redo(IAnjutaEditor* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_REDO, 0, 0);
-}
-
 static int
 itext_editor_get_column(IAnjutaEditor *editor, GError **e)
 {
@@ -2376,12 +2357,6 @@ itext_editor_get_cell_iter (IAnjutaEditor *editor, gint position, GError **e)
 	return IANJUTA_ITERABLE (editor_cell);
 }
 
-static void
-itext_editor_grab_focus (IAnjutaEditor *editor, GError **e)
-{
-	text_editor_grab_focus (TEXT_EDITOR (editor));
-}
-
 static void
 itext_editor_iface_init (IAnjutaEditorIface *iface)
 {
@@ -2394,6 +2369,7 @@ itext_editor_iface_init (IAnjutaEditorIface *iface)
 	iface->goto_position = itext_editor_goto_position;
 	iface->get_text = itext_editor_get_text;
 	iface->get_position = itext_editor_get_position;
+	iface->get_position_iter = itext_editor_get_position_iter;
 	iface->get_lineno = itext_editor_get_lineno;
 	iface->get_length = itext_editor_get_length;
 	iface->get_current_word = itext_editor_get_current_word;
@@ -2401,13 +2377,6 @@ itext_editor_iface_init (IAnjutaEditorIface *iface)
 	iface->append = itext_editor_append;
 	iface->erase = itext_editor_erase;
 	iface->erase_all = itext_editor_erase_all;
-	iface->get_filename = itext_editor_get_filename;
-	iface->can_undo = itext_editor_can_undo;
-	iface->can_redo = itext_editor_can_redo;
-	iface->undo = itext_editor_undo;
-	iface->redo = itext_editor_redo;
-	iface->begin_undo_action = itext_editor_begin_undo_action;
-	iface->end_undo_action = itext_editor_end_undo_action;
 	iface->get_column = itext_editor_get_column;
 	iface->get_overwrite = itext_editor_get_overwrite;
 	iface->set_popup_menu = itext_editor_set_popup_menu;
@@ -2415,7 +2384,97 @@ itext_editor_iface_init (IAnjutaEditorIface *iface)
 	iface->get_line_begin_position = itext_editor_get_line_begin_position;
 	iface->get_line_end_position = itext_editor_get_line_end_position;
 	iface->get_cell_iter = itext_editor_get_cell_iter;
-	iface->grab_focus = itext_editor_grab_focus;
+}
+
+static const gchar *
+idocument_get_filename (IAnjutaDocument *editor, GError **e)
+{
+	return (TEXT_EDITOR (editor))->filename;
+}
+
+static gboolean
+idocument_can_undo(IAnjutaDocument* editor, GError **e)
+{
+	return text_editor_can_undo(TEXT_EDITOR(editor));
+}
+
+static gboolean
+idocument_can_redo(IAnjutaDocument *editor, GError **e)
+{
+	return text_editor_can_redo(TEXT_EDITOR(editor));
+}
+
+static void 
+idocument_undo(IAnjutaDocument* te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_UNDO, 0, 0);
+}
+
+static void 
+idocument_begin_undo_action (IAnjutaDocument* te, GError** ee)
+{
+	scintilla_send_message (SCINTILLA (TEXT_EDITOR (te)->scintilla),
+							SCI_BEGINUNDOACTION, 0, 0);
+}
+
+static void 
+idocument_end_undo_action (IAnjutaDocument* te, GError** ee)
+{
+	scintilla_send_message (SCINTILLA (TEXT_EDITOR (te)->scintilla),
+							SCI_ENDUNDOACTION, 0, 0);
+}
+
+static void 
+idocument_redo(IAnjutaDocument* te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_REDO, 0, 0);
+}
+
+static void
+idocument_grab_focus (IAnjutaDocument *editor, GError **e)
+{
+	text_editor_grab_focus (TEXT_EDITOR (editor));
+}
+
+static void 
+idocument_cut(IAnjutaDocument * te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_CUT, 0, 0);
+}
+
+static void 
+idocument_copy(IAnjutaDocument * te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_COPY, 0, 0);
+}
+
+static void 
+idocument_paste(IAnjutaDocument * te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_PASTE, 0, 0);
+}
+
+static void 
+idocument_clear(IAnjutaDocument * te, GError** ee)
+{
+	text_editor_command(TEXT_EDITOR(te), ANE_CLEAR, 0, 0);
+}
+
+static void
+idocument_iface_init(IAnjutaDocumentIface* iface)
+{
+	iface->get_filename = idocument_get_filename;
+	iface->can_undo = idocument_can_undo;
+	iface->can_redo = idocument_can_redo;
+	iface->undo = idocument_undo;
+	iface->redo = idocument_redo;
+	iface->begin_undo_action = idocument_begin_undo_action;
+	iface->end_undo_action = idocument_end_undo_action;
+	iface->grab_focus = idocument_grab_focus;
+	iface->cut = idocument_cut;
+	iface->copy = idocument_copy;
+	iface->paste = idocument_paste;
+	iface->clear = idocument_clear;
 }
 
 /* IAnjutaEditorSelection implementation */
@@ -2438,18 +2497,67 @@ iselection_set (IAnjutaEditorSelection *editor, gint start, gint end,
 						   SCI_SETSEL, end, start);
 }
 
-static int
-iselection_get_start(IAnjutaEditorSelection *editor, GError **e)
+static void
+iselection_set_iter (IAnjutaEditorSelection* edit, 
+				  IAnjutaEditorCell* istart,
+				  IAnjutaEditorCell* iend,
+				  GError** e)
 {
-	return scintilla_send_message(SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-								  SCI_GETSELECTIONSTART,0,0);
+	TextEditorCell* start = TEXT_EDITOR_CELL (istart);
+	TextEditorCell* end = TEXT_EDITOR_CELL (iend);
+	
+	iselection_set (edit, text_editor_cell_get_position (start),
+					text_editor_cell_get_position (end), FALSE, e);
 }
 
-static int
-iselection_get_end(IAnjutaEditorSelection *editor, GError **e)
+static gboolean
+iselection_has_selection (IAnjutaEditorSelection *editor, GError **e)
 {
-	return scintilla_send_message(SCINTILLA(TEXT_EDITOR(editor)->scintilla),
-								  SCI_GETSELECTIONEND, 0, 0);
+	return text_editor_has_selection (TEXT_EDITOR (editor));
+}
+
+static gint
+iselection_get_start (IAnjutaEditorSelection *editor, GError **e)
+{
+	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
+										  SCI_GETSELECTIONSTART, 0, 0);
+	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
+									   SCI_GETSELECTIONEND, 0, 0);
+	return (start != end)? start: -1;
+}
+
+static IAnjutaIterable*
+iselection_get_start_iter (IAnjutaEditorSelection *edit, GError **e)
+{
+	gint start = iselection_get_start (edit, e);
+	if (start != -1)
+	{
+		return IANJUTA_ITERABLE (text_editor_cell_new (TEXT_EDITOR (edit), start));
+	}
+	else
+		return NULL;
+}
+
+static gint
+iselection_get_end (IAnjutaEditorSelection *editor, GError **e)
+{
+	gint start =  scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
+										  SCI_GETSELECTIONSTART, 0, 0);
+	gint end = scintilla_send_message (SCINTILLA(TEXT_EDITOR(editor)->scintilla),
+									   SCI_GETSELECTIONEND, 0, 0);
+	return (start != end)? end: -1;
+}
+
+static IAnjutaIterable*
+iselection_get_end_iter (IAnjutaEditorSelection *edit, GError **e)
+{
+	gint end = iselection_get_end (edit, e);
+	if (end != -1)
+	{
+		return IANJUTA_ITERABLE (text_editor_cell_new (TEXT_EDITOR (edit), end));
+	}
+	else
+		return NULL;
 }
 
 static void
@@ -2527,46 +2635,22 @@ iselection_select_function(IAnjutaEditorSelection *editor, GError **e)
 	}
 }
 
-static void 
-iselection_cut(IAnjutaEditorSelection* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_CUT, 0, 0);
-}
-
-static void 
-iselection_copy(IAnjutaEditorSelection* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_COPY, 0, 0);
-}
-
-static void 
-iselection_paste(IAnjutaEditorSelection* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_PASTE, 0, 0);
-}
-
-static void 
-iselection_clear(IAnjutaEditorSelection* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_CLEAR, 0, 0);
-}
-
 static void
 iselection_iface_init (IAnjutaEditorSelectionIface *iface)
 {
+	iface->has_selection = iselection_has_selection;
 	iface->get = iselection_get;
 	iface->set = iselection_set;
+	iface->set_iter = iselection_set_iter;
 	iface->get_start = iselection_get_start;
+	iface->get_start_iter = iselection_get_start_iter;
 	iface->get_end = iselection_get_end;
+	iface->get_end_iter = iselection_get_end_iter;
 	iface->replace = iselection_replace;
 	iface->select_all = iselection_select_all;
 	iface->select_to_brace = iselection_select_to_brace;
 	iface->select_block = iselection_select_block;
 	iface->select_function = iselection_select_function;
-	iface->cut = iselection_cut;
-	iface->copy = iselection_copy;
-	iface->paste = iselection_paste;
-	iface->clear = iselection_clear;
 }
 
 /* IAnjutaFile implementation */
@@ -2875,38 +2959,58 @@ ilinemode_iface_init (IAnjutaEditorLineModeIface *iface)
 	iface->fix = ilinemode_fix;
 }
 
-/* IAnjutaEditorAutocomplete implementation */
-static void 
-iautocomplete_activate (IAnjutaEditorAutocomplete* te, GError** ee)
-{
-	text_editor_command(TEXT_EDITOR(te), ANE_COMPLETEWORD, 0, 0);
-}
-
-static void
-iautocomplete_iface_init(IAnjutaEditorAutocompleteIface* iface)
-{
-	iface->activate = iautocomplete_activate;
-}
-
 /* IAnjutaEditorAssist implementation */
-#if 0
-static void
-iassist_add_trigger (IAnjutaEditorAssist* iassist, const gchar* trigger,
-					 IAnjutaEditorAssistContextParser context_parser, GError **err)
-{
-	/* FIXME: Implement new IAnjutaEditorAssist interface */
-}
 
 static void
-iassist_remove_trigger (IAnjutaEditorAssist* iassist, const gchar* trigger, GError **err)
+iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices,
+				 gint position, int char_alignment, GError **err)
 {
-	/* FIXME: Implement new IAnjutaEditorAssist interface */
-}
-
-static void
-iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices, int char_alignment, GError **err)
-{
-	/* FIXME: Implement new IAnjutaEditorAssist interface */
+	GString *words;
+	GList *choice;
+	TextEditor *te = TEXT_EDITOR (iassist);
+	
+	g_return_if_fail (IS_TEXT_EDITOR (te));
+					 
+	if (!choices)
+	{
+		scintilla_send_message (SCINTILLA (te->scintilla), SCI_AUTOCCANCEL,
+								0, 0);
+		scintilla_send_message (SCINTILLA (te->scintilla), SCI_CALLTIPCANCEL,
+								0, 0);
+		return;
+	}
+	
+	words = g_string_sized_new (256);
+	
+	choice = choices;
+	while (choice)
+	{
+		if (choice->data)
+		{
+			if (words->len > 0)
+				g_string_append_c (words, ' ');
+			g_string_append(words, choice->data);
+		}
+		choice = g_list_next (choice);
+	}
+	scintilla_send_message (SCINTILLA (te->scintilla),
+							SCI_AUTOCSETAUTOHIDE, 1, 0);
+	scintilla_send_message (SCINTILLA (te->scintilla),
+							SCI_AUTOCSETDROPRESTOFWORD, 1, 0);
+	scintilla_send_message (SCINTILLA (te->scintilla),
+							SCI_AUTOCSETCANCELATSTART, 0, 0);
+	scintilla_send_message (SCINTILLA (te->scintilla),
+							SCI_AUTOCSETCHOOSESINGLE, 0, 0);
+						 
+	if (char_alignment == 0)
+		scintilla_send_message (SCINTILLA (te->scintilla),
+								SCI_USERLISTSHOW, 1 /* dummy */,
+								(uptr_t) words->str);
+	else
+		scintilla_send_message (SCINTILLA (te->scintilla),
+								SCI_AUTOCSHOW, char_alignment,
+								(uptr_t) words->str);
+	g_string_free (words, TRUE);
 }
 
 static GList*
@@ -2918,32 +3022,84 @@ iassist_get_suggestions (IAnjutaEditorAssist *iassist, const gchar *context, GEr
 }
 
 static void 
-iassist_tip (IAnjutaEditorAssist *iassist, GList* tips,  gint char_alignment, GError **err)
+iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips,
+				   gint position, gint char_alignment, GError **err)
 {
-	/* FIXME: Implement new IAnjutaEditorAssist interface */
+	gint lineno, cur_pos, cur_col, real_pos, real_col;
+	GString *calltip;
+	GList *tip;
+	gint tips_count;
+	TextEditor *te = TEXT_EDITOR (iassist);
+	
+	g_return_if_fail (IS_TEXT_EDITOR (te));
+	g_return_if_fail (tips != NULL);
+	tips_count = g_list_length (tips);
+	g_return_if_fail (tips_count > 0);
+	
+	DEBUG_PRINT ("Number of calltips found %d\n", tips_count);
+	
+	calltip = g_string_sized_new (256);
+
+	tip = tips;
+	while (tip)
+	{
+		if (calltip->len > 0)
+			g_string_append_c (calltip, '\n');
+		g_string_append (calltip, (gchar*) tip->data);
+		tip = g_list_next (tip);
+	}
+/*	
+	if (tips_count > 1)
+	{
+		g_string_prepend_c (calltip, '\001');
+		g_string_append_c (calltip, '\002');
+	}
+*/
+	/* Calculate real calltip position */
+	cur_pos = scintilla_send_message (SCINTILLA (te->scintilla),
+									 SCI_GETCURRENTPOS, 0, 0);
+	lineno = scintilla_send_message (SCINTILLA (te->scintilla),
+									 SCI_LINEFROMPOSITION, cur_pos, 0);
+	cur_col = scintilla_send_message (SCINTILLA (te->scintilla),
+									 SCI_GETCOLUMN, cur_pos, 0);
+	real_col = cur_col - char_alignment;
+	if (real_col < 0)
+		real_col = 0;
+	real_pos = scintilla_send_message (SCINTILLA (te->scintilla),
+									   SCI_FINDCOLUMN, lineno, real_col);
+	scintilla_send_message (SCINTILLA (te->scintilla),
+							SCI_CALLTIPSHOW,
+							real_pos,
+							(uptr_t) calltip->str);
+	g_string_free (calltip, TRUE);
+	/* ContinueCallTip_new(); */
 }
 
 static void
-iassist_react (IAnjutaEditorAssist *iassist, gint selection,  
-			   const gchar *completion, GError **err)
+iassist_cancel_tips (IAnjutaEditorAssist *iassist, GError **err)
 {
+	TextEditor *te = TEXT_EDITOR (iassist);
+	scintilla_send_message (SCINTILLA (te->scintilla), SCI_CALLTIPCANCEL, 0, 0);
+}
 
-	/* FIXME: Implement new IAnjutaEditorAssist interface */
+static void
+iassist_hide_suggestions (IAnjutaEditorAssist *iassist, GError **err)
+{
+	TextEditor *te = TEXT_EDITOR (iassist);
+	
+	g_return_if_fail (IS_TEXT_EDITOR (te));
+	scintilla_send_message (SCINTILLA (te->scintilla), SCI_AUTOCCANCEL, 0, 0);
 }
 
 static void
 iassist_iface_init(IAnjutaEditorAssistIface* iface)
 {
-	iface->autocomplete = iassist_autocomplete;
-	iface->add_trigger = iassist_add_trigger;
-	iface->remove_trigger = iassist_remove_trigger;
-	iface->suggest = iassist_suggest;
 	iface->get_suggestions = iassist_get_suggestions;
-	iface->tip = iassist_tip;
-	iface->react = iassist_react;
+	iface->suggest = iassist_suggest;
+	iface->hide_suggestions = iassist_hide_suggestions;
+	iface->show_tips = iassist_show_tips;
+	iface->cancel_tips = iassist_cancel_tips;
 }
-#endif
-
 
 /* IAnutaEditorFolds implementation */
 
@@ -3386,14 +3542,104 @@ ilanguage_iface_init (IAnjutaEditorLanguageIface *iface)
 	iface->set_language = ilanguage_set_language;
 }
 
+static gboolean
+isearch_forward (IAnjutaEditorSearch* isearch,
+				 const gchar* search,
+				 gboolean case_sensitive,
+				 IAnjutaEditorCell* istart, 
+				 IAnjutaEditorCell* iend,
+				 IAnjutaEditorCell** iresult_start,
+				 IAnjutaEditorCell** iresult_end,
+				 GError** e)
+{
+	TextEditor *te = TEXT_EDITOR (isearch);
+	gint start = text_editor_cell_get_position (TEXT_EDITOR_CELL (istart));
+	gint end = text_editor_cell_get_position (TEXT_EDITOR_CELL (iend));
+
+	gint flags = 0;
+	gint retval;
+	
+	if (case_sensitive)
+	{
+		flags = SCFIND_MATCHCASE;
+	}
+	
+	struct TextToFind to_find;
+	
+	to_find.chrg.cpMin = start;
+	to_find.chrg.cpMax = end;
+	
+	to_find.lpstrText = (gchar*) search;
+	
+	retval = scintilla_send_message (SCINTILLA (te->scintilla),
+									 SCI_FINDTEXT, flags, (long) &to_find);
+	if (retval == -1)
+		return FALSE;
+	else
+	{
+		*iresult_start = IANJUTA_EDITOR_CELL (text_editor_cell_new (te, to_find.chrgText.cpMin));
+		*iresult_end = IANJUTA_EDITOR_CELL (text_editor_cell_new (te, to_find.chrgText.cpMax));
+		return TRUE;
+	}
+}
+
+static gboolean
+isearch_backward (IAnjutaEditorSearch* isearch,
+				  const gchar* search,
+				  gboolean case_sensitive,
+				  IAnjutaEditorCell* istart, 
+				  IAnjutaEditorCell* iend,
+				  IAnjutaEditorCell** iresult_start,
+				  IAnjutaEditorCell** iresult_end,
+				  GError** e)
+{
+	TextEditor *te = TEXT_EDITOR (isearch);
+	gint end = text_editor_cell_get_position (TEXT_EDITOR_CELL (istart));
+	gint start = text_editor_cell_get_position (TEXT_EDITOR_CELL (iend));
+
+	gint flags = 0;
+	gint retval;
+	
+	if (case_sensitive)
+	{
+		flags = SCFIND_MATCHCASE;
+	}
+	
+	struct TextToFind to_find;
+	
+	to_find.chrg.cpMin = start;
+	to_find.chrg.cpMax = end;
+	
+	to_find.lpstrText = (gchar*) search;
+	
+	retval = scintilla_send_message (SCINTILLA (te->scintilla), SCI_FINDTEXT, flags, (long) &to_find);
+	if (retval == -1)
+		return FALSE;
+	else
+	{
+		*iresult_start = IANJUTA_EDITOR_CELL (text_editor_cell_new (te, to_find.chrgText.cpMin));
+		*iresult_end = IANJUTA_EDITOR_CELL (text_editor_cell_new (te, to_find.chrgText.cpMax));
+		return TRUE;
+	}
+}
+
+static void
+isearch_iface_init(IAnjutaEditorSearchIface* iface)
+{
+	iface->forward = isearch_forward;
+	iface->backward = isearch_backward;
+}
+
+
 ANJUTA_TYPE_BEGIN(TextEditor, text_editor, GTK_TYPE_VBOX);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
 ANJUTA_TYPE_ADD_INTERFACE(isavable, IANJUTA_TYPE_FILE_SAVABLE);
+ANJUTA_TYPE_ADD_INTERFACE(idocument, IANJUTA_TYPE_DOCUMENT);
 ANJUTA_TYPE_ADD_INTERFACE(itext_editor, IANJUTA_TYPE_EDITOR);
 ANJUTA_TYPE_ADD_INTERFACE(ilinemode, IANJUTA_TYPE_EDITOR_LINE_MODE);
 ANJUTA_TYPE_ADD_INTERFACE(iselection, IANJUTA_TYPE_EDITOR_SELECTION);
 ANJUTA_TYPE_ADD_INTERFACE(iconvert, IANJUTA_TYPE_EDITOR_CONVERT);
-ANJUTA_TYPE_ADD_INTERFACE(iautocomplete, IANJUTA_TYPE_EDITOR_AUTOCOMPLETE);
+ANJUTA_TYPE_ADD_INTERFACE(iassist, IANJUTA_TYPE_EDITOR_ASSIST);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_ADD_INTERFACE(iview, IANJUTA_TYPE_EDITOR_VIEW);
 ANJUTA_TYPE_ADD_INTERFACE(ifolds, IANJUTA_TYPE_EDITOR_FOLDS);
@@ -3404,6 +3650,7 @@ ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
 ANJUTA_TYPE_ADD_INTERFACE(icomment, IANJUTA_TYPE_EDITOR_COMMENT);
 ANJUTA_TYPE_ADD_INTERFACE(izoom, IANJUTA_TYPE_EDITOR_ZOOM);
 ANJUTA_TYPE_ADD_INTERFACE(igoto, IANJUTA_TYPE_EDITOR_GOTO);
+ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
 
 /* FIXME: Is factory definition really required for editor class? */
 ANJUTA_TYPE_ADD_INTERFACE(itext_editor_factory, IANJUTA_TYPE_EDITOR_FACTORY);
