@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /*
@@ -24,17 +24,20 @@
 
 #include <config.h>
 
-#define DEBUG
+/*#define DEBUG*/
+#include <libanjuta/anjuta-debug.h>
 
 #include "plugin.h"
 
 #include "debugger.h"
 
 #include <libanjuta/interfaces/ianjuta-debugger.h>
+#include <libanjuta/interfaces/ianjuta-debugger-breakpoint.h>
 #include <libanjuta/interfaces/ianjuta-cpu-debugger.h>
 #include <libanjuta/interfaces/ianjuta-variable-debugger.h>
-#include <libanjuta/anjuta-debug.h>
+#include <libanjuta/interfaces/ianjuta-terminal.h>
 #include <libanjuta/anjuta-plugin.h>
+#include <signal.h>
 
 /* Plugin type
  *---------------------------------------------------------------------------*/
@@ -52,6 +55,9 @@ struct _GdbPlugin
 
 	/* Log message window */
 	IAnjutaMessageView *view;
+
+	/* Terminal */
+	pid_t term_pid;
 };
 
 struct _GdbPluginClass
@@ -59,21 +65,127 @@ struct _GdbPluginClass
 	AnjutaPluginClass parent_class;
 };
 
+/* Terminal functions
+ *---------------------------------------------------------------------------*/
+
+static void
+gdb_plugin_stop_terminal (GdbPlugin* plugin)
+{
+	DEBUG_PRINT ("In function: gdb_plugin_stop_terminal()");
+
+	if (plugin->term_pid > 0) {
+		kill (plugin->term_pid, SIGTERM);
+		plugin->term_pid = -1;
+	}
+}
+
+static gchar*
+gdb_plugin_start_terminal (GdbPlugin* plugin)
+{
+	gchar *file, *cmd;
+       	gchar *tty = NULL;
+	IAnjutaTerminal *term;
+
+	DEBUG_PRINT ("In function: gdb_plugin_start_terminal() previous pid %d", plugin->term_pid);
+
+	/* Close previous terminal if needed */
+	gdb_plugin_stop_terminal (plugin);
+
+	/* Get terminal plugin */	
+	term = anjuta_shell_get_interface (ANJUTA_PLUGIN (plugin)->shell, IAnjutaTerminal, NULL);
+	if (term == NULL)
+	{
+
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Anjuta terminal plugin is not installed. The program will be run without a terminal."));
+
+		return NULL;
+	}
+
+	/* Check if anjuta launcher is here */
+	if (anjuta_util_prog_is_installed ("anjuta_launcher", TRUE) == FALSE)
+	{
+		return NULL;
+	}
+
+	file = anjuta_util_get_a_tmp_file();
+	if (mkfifo (file, 0664) < 0)
+	{
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Fail to create fifo file named %s. The program will be run without a terminal"), file);
+		g_free (file);
+
+		return NULL;
+	}
+
+	/* Launch terminal */
+	cmd = g_strconcat ("anjuta_launcher --__debug_terminal ", file, NULL);
+	plugin->term_pid = ianjuta_terminal_execute_command (term, NULL, cmd, NULL);
+	g_free (cmd);
+
+	if (plugin->term_pid > 0)
+	{
+		/*
+		 * Warning: call to fopen() may be blocked if the terminal is
+		 * not properly started. I don't know how to handle this. May
+		 * be opening as non-blocking will solve this .
+		 */
+		g_file_get_contents (file, &tty, NULL, NULL);  /* Ok, take the risk. */
+		if (tty)
+		{       
+			g_strchomp (tty);	/* anjuta_launcher add a end of line after the terminal name */
+			if (strcmp(tty, "__ERROR__") == 0)
+			{
+				g_free (tty);
+				tty = NULL;
+			}
+		}
+	}
+	remove (file);
+	g_free (file);
+
+	if (tty == NULL)
+	{
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
+					  _("Cannot start terminal for debugging."));
+		gdb_plugin_stop_terminal (plugin);
+	}
+
+	return tty;
+}
+
 /* Private functions
  *---------------------------------------------------------------------------*/
 
+static void
+on_debugger_stopped (GdbPlugin *self, GError *err)
+{
+	if (self->debugger != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (self, G_CALLBACK (on_debugger_stopped), self);	
+		debugger_free (self->debugger);
+		self->debugger = NULL;
+		gdb_plugin_stop_terminal (self);
+	}
+}
+
 static void
 gdb_plugin_initialize (GdbPlugin *this)
 {
-	if (this->debugger == NULL)
+	GtkWindow *parent;
+
+	/* debugger can be not NULL, if initialize is called several times
+	 * or if debugger_stopped signal has not been emitted */
+	if (this->debugger != NULL)
 	{
-		GtkWindow *parent;
-		
-		parent = GTK_WINDOW (ANJUTA_PLUGIN (this)->shell);
-		this->debugger = debugger_new (parent, G_OBJECT (this));
-		debugger_set_output_callback (this->debugger, this->output_callback, this->output_user_data);
-		if (this->view) debugger_set_log (this->debugger, this->view);
+		on_debugger_stopped (this, NULL);
 	}
+
+	parent = GTK_WINDOW (ANJUTA_PLUGIN (this)->shell);
+	this->debugger = debugger_new (parent, G_OBJECT (this));
+	g_signal_connect_swapped (this, "debugger-stopped", G_CALLBACK (on_debugger_stopped), this);
+	debugger_set_output_callback (this->debugger, this->output_callback, this->output_user_data);
+	if (this->view) debugger_set_log (this->debugger, this->view);
 }
 
 /* AnjutaPlugin functions
@@ -121,6 +233,7 @@ gdb_plugin_instance_init (GObject* obj)
 
 	this->debugger = NULL;
 	this->output_callback = NULL;
+	this->term_pid = 0;
 }
 
 /* dispose is the first destruction step. It is used to unref object created
@@ -169,8 +282,8 @@ gdb_plugin_class_init (GObjectClass* klass)
 /* Implementation of IAnjutaDebugger interface
  *---------------------------------------------------------------------------*/
 
-static IAnjutaDebuggerStatus
-idebugger_get_status (IAnjutaDebugger *plugin, GError **err)
+static IAnjutaDebuggerState
+idebugger_get_state (IAnjutaDebugger *plugin, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -180,26 +293,14 @@ idebugger_get_status (IAnjutaDebugger *plugin, GError **err)
 	}
 	else
 	{
-		return debugger_get_status (this->debugger);
+		return debugger_get_state (this->debugger);
 	}
 }	
 
 
-static gboolean
-idebugger_initialize (IAnjutaDebugger *plugin, IAnjutaDebuggerOutputCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-
-	this->output_callback = callback;
-	this->output_user_data = user_data;
-	g_signal_emit_by_name (G_OBJECT (plugin), "debugger-ready", IANJUTA_DEBUGGER_STOPPED);
-
-	return TRUE;
-}
-
 static gboolean
 idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_type,
-				const GList *search_dirs, gboolean terminal, GError **err)
+				const GList *search_dirs, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 	gboolean is_libtool = FALSE;
@@ -230,15 +331,7 @@ idebugger_load (IAnjutaDebugger *plugin, const gchar *file, const gchar* mime_ty
 	// Start debugger
 	gdb_plugin_initialize (this);
 	
-	if (!debugger_start (this->debugger, search_dirs, file, is_libtool, terminal))
-	{
-		/* Set error */
-		g_set_error (err, IANJUTA_DEBUGGER_ERROR, IANJUTA_DEBUGGER_UNKNOWN_ERROR, "Unknown error");
-
-		return FALSE;
-	}
-
-	return TRUE;
+	return debugger_start (this->debugger, search_dirs, file, is_libtool);
 }
 
 static gboolean
@@ -258,18 +351,21 @@ idebugger_attach (IAnjutaDebugger *plugin, pid_t pid, const GList *search_dirs,
 
 	// Start debugger
 	gdb_plugin_initialize (this);
-	debugger_start (this->debugger, search_dirs, NULL, FALSE, FALSE);
+	debugger_start (this->debugger, search_dirs, NULL, FALSE);
 	debugger_attach_process (this->debugger, pid);
 	
 	return TRUE;
 }
 
 static gboolean
-idebugger_start (IAnjutaDebugger *plugin, const gchar *argument, GError **err)
+idebugger_start (IAnjutaDebugger *plugin, const gchar *argument, gboolean terminal, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	gchar *tty;
 
-	debugger_start_program (this->debugger, argument);
+	tty = terminal ? gdb_plugin_start_terminal (this) : NULL;
+	debugger_start_program (this->debugger, argument, tty);
+	g_free (tty);
 
 	return TRUE;
 }
@@ -381,97 +477,6 @@ idebugger_interrupt (IAnjutaDebugger *plugin, GError **err)
 	return TRUE;
 }
 
-static gboolean
-idebugger_add_breakpoint_at_line (IAnjutaDebugger *plugin, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (file == NULL) return TRUE;
-		
-	debugger_add_breakpoint_at_line (this->debugger, file, line, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_add_breakpoint_at_function (IAnjutaDebugger *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (function == NULL) return TRUE;
-
-	debugger_add_breakpoint_at_function (this->debugger, *file == '\0' ? NULL : file, function, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_add_breakpoint_at_address (IAnjutaDebugger *plugin, guint address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (address == 0) return TRUE;
-
-	debugger_add_breakpoint_at_address (this->debugger, address, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_enable_breakpoint (IAnjutaDebugger *plugin, guint id, gboolean enable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (id == 0) return TRUE;
-
-	debugger_enable_breakpoint (this->debugger, id, enable, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_ignore_breakpoint (IAnjutaDebugger *plugin, guint id, guint ignore, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (id == 0) return TRUE;
-
-	debugger_ignore_breakpoint (this->debugger, id, ignore, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_condition_breakpoint (IAnjutaDebugger *plugin, guint id, const gchar *condition, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-	
-	/* NULL breakpoint is used to detect available feature */
-	if (id == 0) return TRUE;
-
-	debugger_condition_breakpoint (this->debugger, id, condition, callback, user_data);
-
-	return TRUE;
-}
-
-static gboolean
-idebugger_remove_breakpoint (IAnjutaDebugger *plugin, guint id, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
-{
-	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
-
-	/* NULL breakpoint is used to detect available feature */
-	if (id == 0) return TRUE;
-
-	debugger_remove_breakpoint (this->debugger, id, callback, user_data);
-
-	return TRUE;
-}
-
 static gboolean
 idebugger_inspect (IAnjutaDebugger *plugin, const gchar *name, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
 {
@@ -649,7 +654,7 @@ idebugger_list_frame (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_set_thread (IAnjutaDebugger *plugin, guint thread, GError **err)
+idebugger_set_thread (IAnjutaDebugger *plugin, gint thread, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -669,7 +674,7 @@ idebugger_list_thread (IAnjutaDebugger *plugin, IAnjutaDebuggerCallback callback
 }
 
 static gboolean
-idebugger_info_thread (IAnjutaDebugger *plugin, guint thread, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+idebugger_info_thread (IAnjutaDebugger *plugin, gint thread, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
 	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
 
@@ -720,8 +725,7 @@ idebugger_disable_log (IAnjutaDebugger *plugin, GError **err)
 static void
 idebugger_iface_init (IAnjutaDebuggerIface *iface)
 {
-	iface->get_status = idebugger_get_status;
-	iface->initialize = idebugger_initialize;
+	iface->get_state = idebugger_get_state;
 	iface->attach = idebugger_attach;
 	iface->load = idebugger_load;
 	iface->start = idebugger_start;
@@ -735,13 +739,6 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->run_to = idebugger_run_to;
 	iface->exit = idebugger_exit;
 	iface->interrupt = idebugger_interrupt;
-	iface->set_breakpoint_at_line = idebugger_add_breakpoint_at_line;
-	iface->clear_breakpoint = idebugger_remove_breakpoint;
-	iface->set_breakpoint_at_address = idebugger_add_breakpoint_at_address;
-	iface->set_breakpoint_at_function = idebugger_add_breakpoint_at_function;
-	iface->enable_breakpoint = idebugger_enable_breakpoint;
-	iface->ignore_breakpoint = idebugger_ignore_breakpoint;
-	iface->condition_breakpoint = idebugger_condition_breakpoint;
 
 	iface->inspect = idebugger_inspect;
 	iface->evaluate = idebugger_evaluate;
@@ -773,6 +770,115 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->disable_log = idebugger_disable_log;
 }
 
+
+/* Implementation of IAnjutaDebuggerBreakpoint interface
+ *---------------------------------------------------------------------------*/
+
+static gint
+idebugger_breakpoint_implement (IAnjutaDebuggerBreakpoint *plugin, GError **err)
+{
+	/* gdb implement all interface methods */
+	return IANJUTA_DEBUGGER_BREAKPOINT_SET_AT_ADDRESS
+		| IANJUTA_DEBUGGER_BREAKPOINT_SET_AT_FUNCTION
+		| IANJUTA_DEBUGGER_BREAKPOINT_ENABLE
+		| IANJUTA_DEBUGGER_BREAKPOINT_IGNORE
+		| IANJUTA_DEBUGGER_BREAKPOINT_CONDITION;
+}
+
+static gboolean
+idebugger_breakpoint_add_at_line (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, guint line, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_add_breakpoint_at_line (this->debugger, file, line, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_add_at_function (IAnjutaDebuggerBreakpoint *plugin, const gchar* file, const gchar* function, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_add_breakpoint_at_function (this->debugger, *file == '\0' ? NULL : file, function, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_add_at_address (IAnjutaDebuggerBreakpoint *plugin, guint address, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_add_breakpoint_at_address (this->debugger, address, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_enable (IAnjutaDebuggerBreakpoint *plugin, guint id, gboolean enable, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_enable_breakpoint (this->debugger, id, enable, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_ignore (IAnjutaDebuggerBreakpoint *plugin, guint id, guint ignore, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_ignore_breakpoint (this->debugger, id, ignore, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_condition (IAnjutaDebuggerBreakpoint *plugin, guint id, const gchar *condition, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+	
+	debugger_condition_breakpoint (this->debugger, id, condition, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_remove (IAnjutaDebuggerBreakpoint *plugin, guint id, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_remove_breakpoint (this->debugger, id, callback, user_data);
+
+	return TRUE;
+}
+
+static gboolean
+idebugger_breakpoint_list (IAnjutaDebuggerBreakpoint *plugin, IAnjutaDebuggerCallback callback, gpointer user_data, GError **err)
+{
+	GdbPlugin *this = ANJUTA_PLUGIN_GDB (plugin);
+
+	debugger_list_breakpoint (this->debugger, callback, user_data);
+
+	return TRUE;
+}
+
+static void
+idebugger_breakpoint_iface_init (IAnjutaDebuggerBreakpointIface *iface)
+{
+	iface->implement = idebugger_breakpoint_implement;
+	iface->set_at_line = idebugger_breakpoint_add_at_line;
+	iface->clear = idebugger_breakpoint_remove;
+	iface->list = idebugger_breakpoint_list;
+	iface->set_at_address = idebugger_breakpoint_add_at_address;
+	iface->set_at_function = idebugger_breakpoint_add_at_function;
+	iface->enable = idebugger_breakpoint_enable;
+	iface->ignore = idebugger_breakpoint_ignore;
+	iface->condition = idebugger_breakpoint_condition;
+}
+
 /* Implementation of IAnjutaCpuDebugger interface
  *---------------------------------------------------------------------------*/
 
@@ -912,6 +1018,7 @@ ivariable_debugger_iface_init (IAnjutaVariableDebuggerIface *iface)
 
 ANJUTA_PLUGIN_BEGIN (GdbPlugin, gdb_plugin);
 ANJUTA_PLUGIN_ADD_INTERFACE(idebugger, IANJUTA_TYPE_DEBUGGER);
+ANJUTA_PLUGIN_ADD_INTERFACE(idebugger_breakpoint, IANJUTA_TYPE_DEBUGGER_BREAKPOINT);
 ANJUTA_PLUGIN_ADD_INTERFACE(icpu_debugger, IANJUTA_TYPE_CPU_DEBUGGER);
 ANJUTA_PLUGIN_ADD_INTERFACE(ivariable_debugger, IANJUTA_TYPE_VARIABLE_DEBUGGER);
 ANJUTA_PLUGIN_END;
