@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA 02110-1301, USA. 
  */
  
 /*
@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: anjuta-view.c 2964 2007-05-14 19:58:50Z jhs $ 
+ * $Id: anjuta-view.c 3216 2007-10-14 19:59:37Z jhs $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -381,9 +381,18 @@ static gint
 anjuta_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
 {
 	AnjutaView *view = ANJUTA_VIEW (widget);
+	AssistWindow* assist_win = view->priv->sv->priv->assist_win;
+	AssistTip* assist_tip = view->priv->sv->priv->assist_tip;
+	
+	if (assist_win)
+		gtk_widget_destroy(GTK_WIDGET(assist_win));
+	
+	if (assist_tip)
+		gtk_widget_destroy(GTK_WIDGET(assist_tip));
 	
 	gtk_widget_queue_draw (widget);
 	
+	
 	(* GTK_WIDGET_CLASS (anjuta_view_parent_class)->focus_out_event) (widget, event);
 	
 	return FALSE;
@@ -754,6 +763,8 @@ anjuta_view_key_press_event		(GtkWidget *widget, GdkEventKey       *event)
 	AnjutaView* view = ANJUTA_VIEW(widget);
 	gint pos;
 	GtkTextIter iter;
+	AssistWindow* assist_win;
+	AssistTip* assist_tip;
 	
 	buffer  = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	gtk_text_buffer_get_iter_at_mark(buffer, &iter, 
@@ -761,16 +772,17 @@ anjuta_view_key_press_event		(GtkWidget *widget, GdkEventKey       *event)
 
 	pos = gtk_text_iter_get_offset(&iter);
 
-	AssistWindow* assist_win = view->priv->sv->priv->assist_win;
+	assist_win = view->priv->sv->priv->assist_win;
+	assist_tip = view->priv->sv->priv->assist_tip;
 	if (assist_win)
 	{
 	  if (assist_window_filter_keypress(assist_win, event->keyval))
 	  {
-		DEBUG_PRINT("key filtered: %d", event->keyval);
-		return TRUE;
+			DEBUG_PRINT("key filtered: %d", event->keyval);
+			return TRUE;
 	  }
 	}
-  
+	
 	switch (event->keyval)
 	 {
 		case GDK_Shift_L:
@@ -798,6 +810,22 @@ anjuta_view_key_press_event		(GtkWidget *widget, GdkEventKey       *event)
 				g_signal_emit_by_name(G_OBJECT(view), "char_added", 
 									  pos, '\0');
 			}
+			else if (event->keyval == GDK_Escape || 
+							 event->keyval == GDK_Up ||
+							 event->keyval == GDK_Down)
+			{
+				if (assist_tip)
+					gtk_widget_destroy (GTK_WIDGET(assist_tip));
+			}
+			else if (event->keyval == GDK_Left || event->keyval == GDK_Right ||
+							 event->keyval == GDK_Up || event->keyval == GDK_Down ||
+							 event->keyval == GDK_Page_Up || event->keyval == GDK_Page_Down ||
+							 event->keyval == GDK_Begin || event->keyval == GDK_End)
+			{
+				/* Ignore those for char_added */
+				return retval;
+			}
+							 
 			else
 			{
 				gchar* unistring = g_new0(gchar, 6);
