@@ -20,7 +20,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <libanjuta/anjuta-debug.h>
@@ -32,11 +32,13 @@
 #include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
 #include <libanjuta/interfaces/ianjuta-language-support.h>
+#include <libanjuta/interfaces/ianjuta-document.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-editor-convert.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
+#include <libanjuta/interfaces/ianjuta-editor-search.h>
 
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -66,6 +68,9 @@
 
 #define MONITOR_KEY "sourceview.enable.vfs"
 
+/* globale GtkSourceLanguagesManager instance */
+static GtkSourceLanguagesManager* lang_manager = NULL;
+
 static void sourceview_class_init(SourceviewClass *klass);
 static void sourceview_instance_init(Sourceview *sv);
 static void sourceview_finalize(GObject *object);
@@ -76,20 +81,36 @@ static gboolean sourceview_add_monitor(Sourceview* sv);
 
 /* Callbacks */
 
-static void on_assist_chosen(AssistWindow* assist_win, gint num, Sourceview* sv)
+static void
+on_assist_window_destroyed (AssistWindow* window, Sourceview* sv)
+{
+	sv->priv->assist_win = NULL;
+	DEBUG_PRINT("Assist-win-destroy");
+}
+
+static void 
+on_assist_tip_destroyed (AssistTip* tip, Sourceview* sv)
+{
+	sv->priv->assist_tip = NULL;
+	DEBUG_PRINT("Assist-tip-destroy");
+}
+
+static void 
+on_assist_chosen(AssistWindow* assist_win, gint num, Sourceview* sv)
 {
 	g_signal_emit_by_name(G_OBJECT(sv), "assist_chosen", num);
 	DEBUG_PRINT("Assist-choosen");
-	gtk_widget_hide(GTK_WIDGET(sv->priv->assist_win));
-	gtk_widget_destroy(GTK_WIDGET(sv->priv->assist_win));
 }
 
-static void on_assist_cancel(AssistWindow* assist_win, Sourceview* sv)
+static void 
+on_assist_cancel(AssistWindow* assist_win, Sourceview* sv)
 {
 	g_signal_emit_by_name(G_OBJECT(sv), "assist_canceled");
 	DEBUG_PRINT("Assist-cancel");
-	gtk_widget_hide(GTK_WIDGET(sv->priv->assist_win));
-	gtk_widget_destroy(GTK_WIDGET(sv->priv->assist_win));
+	if (sv->priv->assist_win)
+	{
+		gtk_widget_destroy(GTK_WIDGET(sv->priv->assist_win));
+	}
 }
 
 /* Called when a character is added */
@@ -97,70 +118,6 @@ static void on_document_char_added(AnjutaView* view, gint pos,
 								   gchar ch,
 								   Sourceview* sv)
 {
-	gint i;
-	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-	GtkTextIter begin;
-	GtkTextIter end;
-	DEBUG_PRINT("on_document_char_added, assist: %d", sv->priv->assist_win != NULL);
-	if (sv->priv->assist_win)
-	{
-		gtk_text_buffer_get_iter_at_mark(buffer, &begin,
-										 gtk_text_buffer_get_insert(buffer));
-		gtk_text_buffer_get_iter_at_offset(buffer, &end,
-										   assist_window_get_position(sv->priv->assist_win));
-		gchar* context = gtk_text_buffer_get_text(buffer, &begin, &end, FALSE);
-		g_signal_emit_by_name(G_OBJECT(sv), "assist_update", context);
-	}
-	/* FIXME: triggers with more than 5 characters? */
-	else
-	{
-		gboolean found = FALSE;
-		for (i = 1; i < 5; i++)
-		{
-			gchar* text;
-			IAnjutaEditorAssistContextParser parser = 0;
-			gtk_text_buffer_get_iter_at_mark(buffer, &begin, gtk_text_buffer_get_insert(buffer));
-			gtk_text_buffer_get_iter_at_mark(buffer, &end, gtk_text_buffer_get_insert(buffer));		
-			gtk_text_iter_backward_chars(&begin, i);
-			text = gtk_text_buffer_get_text(buffer, &begin, &end, FALSE);
-			parser = g_hash_table_lookup(sv->priv->triggers, text);
-			if (parser)
-			{
-				gchar* context = parser(IANJUTA_EDITOR(sv), gtk_text_iter_get_offset(&end));
-				sv->priv->assist_win = assist_window_new(GTK_TEXT_VIEW(sv->priv->view), text, -1);
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "destroy", 
-								 G_CALLBACK(gtk_widget_destroyed), &sv->priv->assist_win);
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "chosen", 
-								 G_CALLBACK(on_assist_chosen), sv);
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "cancel", 
-								 G_CALLBACK(on_assist_cancel), sv);
-				g_signal_emit_by_name(G_OBJECT(sv), "assist_begin", context, text);
-				g_free(text);
-				found = TRUE;
-				break;
-			}
-			g_free(text);
-		}
-		if (!found)
-		{
-			gchar* word = anjuta_document_get_current_word(
-														   ANJUTA_DOCUMENT(sv->priv->document));
-			if (word != NULL && strlen(word) >= 3)
-			{
-				gtk_text_buffer_get_iter_at_mark(buffer, &begin, gtk_text_buffer_get_insert(buffer));
-				gtk_text_iter_backward_chars(&begin, strlen(word));
-				sv->priv->assist_win = assist_window_new(GTK_TEXT_VIEW(sv->priv->view), NULL,
-														 gtk_text_iter_get_offset(&begin));
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "destroy", 
-							 G_CALLBACK(gtk_widget_destroyed), &sv->priv->assist_win);
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "chosen", 
-							 G_CALLBACK(on_assist_chosen), sv);
-				g_signal_connect(G_OBJECT(sv->priv->assist_win), "cancel", 
-							 G_CALLBACK(on_assist_cancel), sv);
-				g_signal_emit_by_name(G_OBJECT(sv), "assist_begin", word, NULL);
-			}
-		}	
-	}
 	if (ch != '\0')
 		g_signal_emit_by_name(G_OBJECT(sv), "char_added", pos, ch);
 }
@@ -223,7 +180,7 @@ on_sourceview_uri_changed_prompt (Sourceview* sv)
 		g_strdup_printf (_
 						 ("The file '%s' on the disk is more recent than\n"
 						  "the current buffer.\nDo you want to reload it?"),
-						 ianjuta_editor_get_filename(IANJUTA_EDITOR(sv), NULL));
+						 ianjuta_document_get_filename(IANJUTA_DOCUMENT(sv), NULL));
 	
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (sv));
 	
@@ -319,6 +276,9 @@ static void on_document_loaded(AnjutaDocument* doc, GError* err, Sourceview* sv)
 	
 	sourceview_add_monitor(sv);
 
+	/* Autodetect language */
+	ianjuta_editor_language_set_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL, NULL);
+
 	lang = ianjuta_editor_language_get_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL);
 	g_signal_emit_by_name (sv, "language-changed", lang);
 	
@@ -422,6 +382,7 @@ static void on_document_saved(AnjutaDocument* doc, GError* err, Sourceview* sv)
 	}
 	else
 	{
+		const gchar* lang;
 		gtk_text_buffer_set_modified(GTK_TEXT_BUFFER(doc), FALSE);
 		g_signal_emit_by_name(G_OBJECT(sv), "save_point", TRUE);
 		/* Set up 2 sec timer */
@@ -430,6 +391,10 @@ static void on_document_saved(AnjutaDocument* doc, GError* err, Sourceview* sv)
 		sv->priv->monitor_delay = g_timeout_add (2000,
 							(GSourceFunc)sourceview_add_monitor, sv);
 		sv->priv->saving = FALSE;
+		/* Autodetect language */
+		ianjuta_editor_language_set_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL, NULL);
+		lang = ianjuta_editor_language_get_language(IANJUTA_EDITOR_LANGUAGE(sv), NULL);
+		g_signal_emit_by_name (sv, "language-changed", lang);
 	}
 	g_timeout_add(3000, (GSourceFunc)timeout_unref, sv);
 }
@@ -438,6 +403,8 @@ static void
 sourceview_instance_init(Sourceview* sv)
 {
 	sv->priv = g_new0(SourceviewPrivate, 1);
+	if (!lang_manager)
+		lang_manager = gtk_source_languages_manager_new();
 }
 
 static void
@@ -460,6 +427,8 @@ sourceview_dispose(GObject *object)
 	Sourceview *cobj = ANJUTA_SOURCEVIEW(object);
 	if (cobj->priv->assist_win)
 		on_assist_cancel(cobj->priv->assist_win, cobj);
+	if (cobj->priv->assist_tip)
+		gtk_widget_destroy(GTK_WIDGET(cobj->priv->assist_tip));
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -482,12 +451,12 @@ sourceview_finalize(GObject *object)
 
 /* Sync with IANJUTA_MARKABLE_MARKER  */
 
-#define MARKER_PIXMAP_LINEMARKER "linemarker.png"
-#define MARKER_PIXMAP_PROGRAM_COUNTER "program-counter.png"
-#define MARKER_PIXMAP_BREAKPOINT_DISABLED "breakpoint-disabled.png"
-#define MARKER_PIXMAP_BREAKPOINT_ENABLED "breakpoint-enabled.png"
+#define MARKER_PIXMAP_LINEMARKER "anjuta-linemark-16.png"
+#define MARKER_PIXMAP_PROGRAM_COUNTER "anjuta-pcmark-16.png"
+#define MARKER_PIXMAP_BREAKPOINT_DISABLED "anjuta-breakpoint-disabled-16.png"
+#define MARKER_PIXMAP_BREAKPOINT_ENABLED "anjuta-breakpoint-enabled-16.png"
 
-#define MARKER_PIXMAP_BOOKMARK "bookmark.png"
+#define MARKER_PIXMAP_BOOKMARK "anjuta-bookmark-16.png"
 
 #define MARKER_NONE "sv-none"
 #define MARKER_LINEMARKER "sv-linemarker"
@@ -548,7 +517,7 @@ static void sourceview_create_highligth_indic(Sourceview* sv)
 	sv->priv->important_indic = 
 		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
 		IMPORTANT_INDIC,
-		"foreground", "#0000FF", NULL);  
+		"background", "#FFFF00", NULL);  
 	sv->priv->warning_indic = 
 		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER(sv->priv->document),
 		WARNING_INDIC,
@@ -720,12 +689,6 @@ ifile_iface_init (IAnjutaFileIface *iface)
 
 /* IAnjutaEditor interface */
 
-/* Grab focus */
-static void ieditor_grab_focus (IAnjutaEditor *editor, GError **e)
-{
-	gtk_widget_grab_focus (GTK_WIDGET (ANJUTA_SOURCEVIEW (editor)->priv->view));
-}
-
 static gint
 ieditor_get_tab_size (IAnjutaEditor *editor, GError **e)
 {
@@ -827,6 +790,21 @@ static gint ieditor_get_position(IAnjutaEditor* editor, GError **e)
 	return gtk_text_iter_get_offset(&iter);
 }
 
+static IAnjutaIterable*
+ieditor_get_position_iter (IAnjutaEditor* editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
+	GtkTextIter iter;
+	SourceviewCell* cell;
+	
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, 
+									 gtk_text_buffer_get_insert(buffer));
+
+	cell = sourceview_cell_new (&iter, GTK_TEXT_VIEW (sv->priv->view));
+	
+	return IANJUTA_ITERABLE (cell);
+}
 
 /* Return line of cursor */
 static gint ieditor_get_lineno(IAnjutaEditor *editor, GError **e)
@@ -868,7 +846,7 @@ static gint ieditor_get_length(IAnjutaEditor *editor, GError **e)
 static gchar* ieditor_get_current_word(IAnjutaEditor *editor, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
-	return anjuta_document_get_current_word(sv->priv->document);
+	return anjuta_document_get_current_word(sv->priv->document, FALSE);
 }
 
 /* Insert text at position */
@@ -922,34 +900,6 @@ static void ieditor_erase_all(IAnjutaEditor *editor, GError **e)
 	gtk_text_buffer_set_text(GTK_TEXT_BUFFER(sv->priv->document), "", 0);
 }
 
-/* Return true if editor can redo */
-static gboolean ieditor_can_redo(IAnjutaEditor *editor, GError **e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
-	return gtk_source_buffer_can_redo(GTK_SOURCE_BUFFER(sv->priv->document));
-}
-
-/* Return true if editor can undo */
-static gboolean ieditor_can_undo(IAnjutaEditor *editor, GError **e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
-	return gtk_source_buffer_can_undo(GTK_SOURCE_BUFFER(sv->priv->document));
-}
-
-/* Return true if editor can undo */
-static void ieditor_begin_undo_action (IAnjutaEditor *editor, GError **e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
-	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER(sv->priv->document));
-}
-
-/* Return true if editor can undo */
-static void ieditor_end_undo_action (IAnjutaEditor *editor, GError **e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
-	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER(sv->priv->document));
-}
-
 /* Return column of cursor */
 static gint ieditor_get_column(IAnjutaEditor *editor, GError **e)
 {
@@ -978,16 +928,6 @@ static void ieditor_set_popup_menu(IAnjutaEditor *editor,
     g_object_set(G_OBJECT(sv->priv->view), "popup", menu, NULL);
 }
 
-/* Return the opened filename */
-static const gchar* ieditor_get_filename(IAnjutaEditor *editor, GError **e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
-	if (sv->priv->filename != NULL)
-		return sv->priv->filename;
-	else
-		return anjuta_document_get_short_name_for_display(sv->priv->document);
-}
-
 /* Convert from position to line */
 static gint ieditor_get_line_from_position(IAnjutaEditor *editor, 
 										   gint position, GError **e)
@@ -1029,25 +969,6 @@ static gint ieditor_get_line_end_position(IAnjutaEditor *editor,
 	return gtk_text_iter_get_offset(&iter);
 }
 
-static void 
-ieditor_undo(IAnjutaEditor* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	if (ieditor_can_undo(edit, NULL))	
-		gtk_source_buffer_undo(GTK_SOURCE_BUFFER(sv->priv->document));
-	anjuta_view_scroll_to_cursor(sv->priv->view);
-	g_signal_emit_by_name(G_OBJECT(sv), "update_ui", sv); 
-}
-
-static void 
-ieditor_redo(IAnjutaEditor* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	if (ieditor_can_redo(edit, NULL))	
-		gtk_source_buffer_redo(GTK_SOURCE_BUFFER(sv->priv->document));
-	anjuta_view_scroll_to_cursor(sv->priv->view);
-}
-
 static IAnjutaIterable*
 ieditor_get_cell_iter(IAnjutaEditor* edit, gint position, GError** e)
 {
@@ -1065,7 +986,6 @@ ieditor_get_cell_iter(IAnjutaEditor* edit, gint position, GError** e)
 static void
 ieditor_iface_init (IAnjutaEditorIface *iface)
 {
-	iface->grab_focus = ieditor_grab_focus;
 	iface->get_tabsize = ieditor_get_tab_size;
 	iface->set_tabsize = ieditor_set_tab_size;
 	iface->get_use_spaces = ieditor_get_use_spaces;
@@ -1075,6 +995,7 @@ ieditor_iface_init (IAnjutaEditorIface *iface)
 	iface->goto_position = ieditor_goto_position;
 	iface->get_text = ieditor_get_text;
 	iface->get_position = ieditor_get_position;
+	iface->get_position_iter = ieditor_get_position_iter;
 	iface->get_lineno = ieditor_get_lineno;
 	iface->get_length = ieditor_get_length;
 	iface->get_current_word = ieditor_get_current_word;
@@ -1082,32 +1003,141 @@ ieditor_iface_init (IAnjutaEditorIface *iface)
 	iface->append = ieditor_append;
 	iface->erase = ieditor_erase;
 	iface->erase_all = ieditor_erase_all;
-	iface->get_filename = ieditor_get_filename;
-	iface->can_undo = ieditor_can_undo;
-	iface->can_redo = ieditor_can_redo;
-	iface->begin_undo_action = ieditor_begin_undo_action;
-	iface->end_undo_action = ieditor_end_undo_action;
 	iface->get_column = ieditor_get_column;
 	iface->get_overwrite = ieditor_get_overwrite;
 	iface->set_popup_menu = ieditor_set_popup_menu;
 	iface->get_line_from_position = ieditor_get_line_from_position;
-	iface->undo = ieditor_undo;
-	iface->redo = ieditor_redo;
 	iface->get_cell_iter = ieditor_get_cell_iter;
 	iface->get_line_begin_position = ieditor_get_line_begin_position;
 	iface->get_line_end_position = ieditor_get_line_end_position;	
 }
 
+/* Return true if editor can redo */
+static gboolean idocument_can_redo(IAnjutaDocument *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	return gtk_source_buffer_can_redo(GTK_SOURCE_BUFFER(sv->priv->document));
+}
+
+/* Return true if editor can undo */
+static gboolean idocument_can_undo(IAnjutaDocument *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
+	return gtk_source_buffer_can_undo(GTK_SOURCE_BUFFER(sv->priv->document));
+}
+
+/* Return true if editor can undo */
+static void idocument_begin_undo_action (IAnjutaDocument *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
+	gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER(sv->priv->document));
+}
+
+/* Return true if editor can undo */
+static void idocument_end_undo_action (IAnjutaDocument *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);	
+	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER(sv->priv->document));
+}
+
+
+static void 
+idocument_undo(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	if (idocument_can_undo(edit, NULL))	
+		gtk_source_buffer_undo(GTK_SOURCE_BUFFER(sv->priv->document));
+	anjuta_view_scroll_to_cursor(sv->priv->view);
+	g_signal_emit_by_name(G_OBJECT(sv), "update_ui", sv); 
+}
+
+static void 
+idocument_redo(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	if (idocument_can_redo(edit, NULL))	
+		gtk_source_buffer_redo(GTK_SOURCE_BUFFER(sv->priv->document));
+	anjuta_view_scroll_to_cursor(sv->priv->view);
+}
+
+/* Grab focus */
+static void idocument_grab_focus (IAnjutaDocument *editor, GError **e)
+{
+	gtk_widget_grab_focus (GTK_WIDGET (ANJUTA_SOURCEVIEW (editor)->priv->view));
+}
+
+/* Return the opened filename */
+static const gchar* idocument_get_filename(IAnjutaDocument *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	if (sv->priv->filename != NULL)
+		return sv->priv->filename;
+	else
+		return anjuta_document_get_short_name_for_display(sv->priv->document);
+}
+
+static void 
+idocument_cut(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	anjuta_view_cut_clipboard(sv->priv->view);
+}
+
+static void 
+idocument_copy(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	anjuta_view_copy_clipboard(sv->priv->view);
+}
+
+static void 
+idocument_paste(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	anjuta_view_paste_clipboard(sv->priv->view);
+}
+
+static void 
+idocument_clear(IAnjutaDocument* edit, GError** ee)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	if (gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(sv->priv->document)))
+		anjuta_view_delete_selection(sv->priv->view);
+	else
+	{
+		GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
+		GtkTextIter cursor;
+		gtk_text_buffer_get_iter_at_mark(buffer, &cursor, gtk_text_buffer_get_insert(buffer));
+		
+		/* Fix #388731 */
+		gtk_text_iter_forward_char(&cursor);
+		gtk_text_buffer_backspace(buffer, &cursor, TRUE, TRUE);
+	}
+}
+
+static void
+idocument_iface_init (IAnjutaDocumentIface *iface)
+{
+	iface->grab_focus = idocument_grab_focus;
+	iface->get_filename = idocument_get_filename;
+	iface->can_undo = idocument_can_undo;
+	iface->can_redo = idocument_can_redo;
+	iface->begin_undo_action = idocument_begin_undo_action;
+	iface->end_undo_action = idocument_end_undo_action;
+	iface->undo = idocument_undo;
+	iface->redo = idocument_redo;
+	iface->cut = idocument_cut;
+	iface->copy = idocument_copy;
+	iface->paste = idocument_paste;
+	iface->clear = idocument_clear;
+}
+
 static void
 set_select(Sourceview* sv, GtkTextIter* start_iter, GtkTextIter* end_iter)
 {
-	gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(sv->priv->document),
-									  "insert", start_iter);
-	gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(sv->priv->document),
-									  "selection_bound", end_iter);
-	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(sv->priv->view),
-								 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)),
-								 0,  FALSE, 0, 0);			
+	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (sv->priv->document), start_iter, end_iter);
+	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(sv->priv->view),
+								 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)));
 }
 
 /* IAnjutaEditorSelection */
@@ -1205,6 +1235,22 @@ static void iselect_set(IAnjutaEditorSelection *editor, gint start,
 	set_select(sv, &start_iter, &end_iter);
 }
 
+static void
+iselect_set_iter (IAnjutaEditorSelection* edit, 
+				  IAnjutaEditorCell* istart,
+				  IAnjutaEditorCell* iend,
+				  GError** e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (sv->priv->document),
+								  sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
+								  sourceview_cell_get_iter (SOURCEVIEW_CELL (iend)));
+	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (sv->priv->view),
+												 sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
+												 0, FALSE, 0, 0);
+}
+															
+
 static void
 iselect_all(IAnjutaEditorSelection* edit, GError** e)
 {	
@@ -1212,6 +1258,14 @@ iselect_all(IAnjutaEditorSelection* edit, GError** e)
 	anjuta_view_select_all(sv->priv->view);
 }
 
+static gboolean
+iselect_has_selection (IAnjutaEditorSelection *editor, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	
+	return gtk_text_buffer_get_has_selection (GTK_TEXT_BUFFER(sv->priv->document));
+}
+
 /* Return a newly allocated pointer containing selected text or
 NULL if no text is selected */
 static gchar* iselect_get(IAnjutaEditorSelection* editor, GError **e)
@@ -1234,22 +1288,56 @@ static gchar* iselect_get(IAnjutaEditorSelection* editor, GError **e)
 /* Get start point of selection */
 static gint iselect_get_start(IAnjutaEditorSelection *editor, GError **e)
 {
-	/* This is the same within gtk_text_buffer */
-	return ianjuta_editor_get_position(IANJUTA_EDITOR(editor), e);
+	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
+	GtkTextIter start;
+	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (sv->priv->document),
+											  &start, NULL))
+	{
+		return gtk_text_iter_get_offset (&start);
+	}
+	return -1;										  
+}
+
+static IAnjutaIterable*
+iselect_get_start_iter (IAnjutaEditorSelection *edit, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	GtkTextIter start;
+	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (sv->priv->document),
+											  &start, NULL))
+	{
+		return IANJUTA_ITERABLE (sourceview_cell_new (&start, GTK_TEXT_VIEW (sv->priv->view)));
+	}
+	return NULL;
 }
 
 /* Get end position of selection */
 static gint iselect_get_end(IAnjutaEditorSelection *editor, GError **e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
-	GtkTextIter end_iter;
-	
-	gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(sv->priv->document),
-									 &end_iter,
-									 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)));
-	return gtk_text_iter_get_offset(&end_iter);
+	GtkTextIter end;
+	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (sv->priv->document),
+											  NULL, &end))
+	{
+		return gtk_text_iter_get_offset (&end);
+	}
+	return -1;
 }
 
+static IAnjutaIterable*
+iselect_get_end_iter (IAnjutaEditorSelection *edit, GError **e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
+	GtkTextIter end;
+	if (gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (sv->priv->document),
+											  NULL, &end))
+	{
+		return IANJUTA_ITERABLE (sourceview_cell_new (&end, GTK_TEXT_VIEW (sv->priv->view)));
+	}
+	return NULL;
+}
+
+
 static void iselect_function(IAnjutaEditorSelection *editor, GError **e)
 {
 	// TODO
@@ -1278,51 +1366,16 @@ static void iselect_replace(IAnjutaEditorSelection* editor,
 	}
 }
 
-static void 
-iselect_cut(IAnjutaEditorSelection* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	anjuta_view_cut_clipboard(sv->priv->view);
-}
-
-static void 
-iselect_copy(IAnjutaEditorSelection* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	anjuta_view_copy_clipboard(sv->priv->view);
-}
-
-static void 
-iselect_paste(IAnjutaEditorSelection* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	anjuta_view_paste_clipboard(sv->priv->view);
-}
-
-static void 
-iselect_clear(IAnjutaEditorSelection* edit, GError** ee)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	if (gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(sv->priv->document)))
-		anjuta_view_delete_selection(sv->priv->view);
-	else
-	{
-		GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-		GtkTextIter cursor;
-		gtk_text_buffer_get_iter_at_mark(buffer, &cursor, gtk_text_buffer_get_insert(buffer));
-		
-		/* Fix #388731 */
-		gtk_text_iter_forward_char(&cursor);
-		gtk_text_buffer_backspace(buffer, &cursor, TRUE, TRUE);
-	}
-}
-
 static void
 iselect_iface_init(IAnjutaEditorSelectionIface *iface)
 {
 	iface->set = iselect_set;
+	iface->set_iter = iselect_set_iter;
+	iface->has_selection = iselect_has_selection;
 	iface->get_start = iselect_get_start;
+	iface->get_start_iter = iselect_get_start_iter;
 	iface->get_end = iselect_get_end;
+	iface->get_end_iter = iselect_get_end_iter;
 	iface->select_block = iselect_block;
 	iface->select_function = iselect_function;
 	iface->select_all = iselect_all;
@@ -1330,10 +1383,6 @@ iselect_iface_init(IAnjutaEditorSelectionIface *iface)
 	iface->select_block = iselect_block;
 	iface->get = iselect_get;
 	iface->replace = iselect_replace;
-	iface->cut = iselect_cut;
-	iface->copy = iselect_copy;
-	iface->paste = iselect_paste;
-	iface->clear = iselect_clear;
 }
 
 /* IAnjutaEditorConvert Interface */
@@ -1755,8 +1804,7 @@ static const GList*
 ilanguage_get_supported_languages (IAnjutaEditorLanguage *ilanguage,
 								   GError **err)
 {
-	GtkSourceLanguagesManager* manager = gtk_source_languages_manager_new();
-	const GSList* manager_languages = gtk_source_languages_manager_get_available_languages(manager);
+	const GSList* manager_languages = gtk_source_languages_manager_get_available_languages(lang_manager);
 	GList* languages = NULL;
 	
 	while (manager_languages)
@@ -1775,16 +1823,33 @@ ilanguage_get_language_name (IAnjutaEditorLanguage *ilanguage,
 	return language;
 }
 
+static const gchar*
+autodetect_language (Sourceview* sv)
+{
+	const gchar* filename = idocument_get_filename (IANJUTA_DOCUMENT(sv), NULL);
+	const gchar* mime_type = gnome_vfs_get_mime_type_for_name (filename);
+	GtkSourceLanguage* lang =
+		gtk_source_languages_manager_get_language_from_mime_type(lang_manager, mime_type);
+	if (lang != NULL)
+	{
+		const gchar* language = gtk_source_language_get_name(lang);
+		gtk_source_buffer_set_language(GTK_SOURCE_BUFFER(sv->priv->document), lang);
+		g_signal_emit_by_name (G_OBJECT(sv), "language-changed", 
+							   language);
+		return language;
+	}
+	return NULL;
+}
+
 static void
 ilanguage_set_language (IAnjutaEditorLanguage *ilanguage,
 						const gchar *language, GError **err)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(ilanguage);
 	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
-	GtkSourceLanguagesManager* manager = gtk_source_languages_manager_new();
-	const GSList* langs = gtk_source_languages_manager_get_available_languages(manager);
+	const GSList* langs = gtk_source_languages_manager_get_available_languages(lang_manager);
 	
-	while (langs)
+	while (language && langs)
 	{
 		gchar* name = gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(langs->data));
 		if (g_str_equal(name, language))
@@ -1795,18 +1860,8 @@ ilanguage_set_language (IAnjutaEditorLanguage *ilanguage,
 		}
 		langs = g_slist_next(langs);
 	}
-	/* Language not found -> use autodetection */
-	{
-		gchar* mime_type = anjuta_document_get_mime_type(ANJUTA_DOCUMENT(buffer));
-		GtkSourceLanguage* lang = gtk_source_languages_manager_get_language_from_mime_type(
-			manager, mime_type);
-		if (lang != NULL)
-		{
-			gtk_source_buffer_set_language(buffer, lang);
-			g_signal_emit_by_name (ilanguage, "language-changed", 
-				gtk_source_language_get_name(lang));
-		}
-	}	
+	/* Otherwise, autodetect language */
+	autodetect_language (sv);
 }
 
 static const gchar*
@@ -1816,9 +1871,14 @@ ilanguage_get_language (IAnjutaEditorLanguage *ilanguage, GError **err)
 	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
 	GtkSourceLanguage* lang = gtk_source_buffer_get_language(buffer);
 	if (lang)
+	{
 		return gtk_source_language_get_name(lang);
+	}
 	else
-		return NULL;
+	{
+		const gchar* language = autodetect_language (sv);
+		return language;
+	}
 }
 
 static void
@@ -1830,93 +1890,212 @@ ilanguage_iface_init (IAnjutaEditorLanguageIface *iface)
 	iface->set_language = ilanguage_set_language;
 }
 
-static void
-iassist_add_trigger (IAnjutaEditorAssist* iassist, const gchar* trigger,
-					 IAnjutaEditorAssistContextParser context_parser, GError **err)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
-	if (!sv->priv->triggers)
-	{
-		sv->priv->triggers = g_hash_table_new(g_str_hash, g_str_equal);
-	}
-	g_hash_table_insert(sv->priv->triggers, (gchar*) trigger, context_parser);
-}
-
-static void
-iassist_remove_trigger (IAnjutaEditorAssist* iassist, const gchar* trigger, GError **err)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
-	if (sv->priv->triggers)
-	{
-		g_hash_table_remove(sv->priv->triggers, trigger);
-	}
-}
-
 static GList*
 iassist_get_suggestions (IAnjutaEditorAssist *iassist, const gchar *context, GError **err)
 {
-	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
+	/* Sourceview* sv = ANJUTA_SOURCEVIEW(iassist); */
 	/* We don't make own suggestions yet */
 	return NULL;
 }
 
 static void
-iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices, int char_alignment, GError **err)
+iassist_suggest (IAnjutaEditorAssist *iassist, GList* choices, int position,
+				 int char_alignment, GError **err)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
-	g_return_if_fail(sv->priv->assist_win != NULL);
+	
 	if (choices == NULL)
 	{
-		g_signal_emit_by_name(G_OBJECT(sv), "assist_end");
-		gtk_widget_destroy(GTK_WIDGET(sv->priv->assist_win));
-		sv->priv->assist_win = NULL;
+		if (sv->priv->assist_win)
+			gtk_widget_destroy(GTK_WIDGET(sv->priv->assist_win));
 	}
 	else
 	{
+		if (!sv->priv->assist_win)
+		{
+			sv->priv->assist_win = assist_window_new(GTK_TEXT_VIEW(sv->priv->view), NULL,
+													 position);
+			g_signal_connect(G_OBJECT(sv->priv->assist_win), "destroy", 
+								 G_CALLBACK(on_assist_window_destroyed), sv);
+			g_signal_connect(G_OBJECT(sv->priv->assist_win), "chosen", 
+								 G_CALLBACK(on_assist_chosen), sv);
+			g_signal_connect(G_OBJECT(sv->priv->assist_win), "cancel", 
+								 G_CALLBACK(on_assist_cancel), sv);
+		}
 		assist_window_update(sv->priv->assist_win, choices);
 		gtk_widget_show(GTK_WIDGET(sv->priv->assist_win));
-		if (char_alignment != -1)
-			assist_window_move(sv->priv->assist_win, char_alignment);
+		if (char_alignment > 0)
+		{
+			/* Calculate offset */
+			GtkTextIter cursor;
+			GtkTextBuffer* buffer = GTK_TEXT_BUFFER (sv->priv->document);
+			gtk_text_buffer_get_iter_at_mark (buffer,
+											  &cursor,
+											  gtk_text_buffer_get_insert(buffer));
+			
+			gint offset = gtk_text_iter_get_offset (&cursor);
+			assist_window_move(sv->priv->assist_win, offset - char_alignment);
+		}
+	}
+}
+
+static void
+iassist_hide_suggestions (IAnjutaEditorAssist* iassist, GError** err)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
+	if (sv->priv->assist_win)
+	{
+		gtk_widget_hide (GTK_WIDGET (sv->priv->assist_win));
 	}
 }
 
 static void 
-iassist_tip (IAnjutaEditorAssist *iassist, GList* tips,  gint char_alignment, GError **err)
+iassist_show_tips (IAnjutaEditorAssist *iassist, GList* tips, gint position,
+				   gint char_alignment, GError **err)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
+	int tip_position = position - char_alignment;
+	
+	if (!sv->priv->assist_tip)
+	{
+		sv->priv->assist_tip = 
+			ASSIST_TIP (assist_tip_new (GTK_TEXT_VIEW (sv->priv->view), tips));
+		
+		g_signal_connect (G_OBJECT(sv->priv->assist_tip), "destroy", G_CALLBACK(on_assist_tip_destroyed),
+						  sv);
+		assist_tip_move (sv->priv->assist_tip, GTK_TEXT_VIEW (sv->priv->view), tip_position);
+		gtk_widget_show (GTK_WIDGET (sv->priv->assist_tip));
+	}
+	else
+	{
+		assist_tip_set_tips (sv->priv->assist_tip, tips);
+		assist_tip_move (sv->priv->assist_tip, GTK_TEXT_VIEW (sv->priv->view), tip_position);
+	}
 }
 
 static void
-iassist_react (IAnjutaEditorAssist *iassist, gint selection,  
-			   const gchar *completion, GError **err)
+iassist_cancel_tips (IAnjutaEditorAssist* iassist, GError** err)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(iassist);
-	GtkTextIter begin, end;
-	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
-															   &begin, assist_window_get_position(sv->priv->assist_win));
-	gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(sv->priv->document), &end, 
-										   gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)));
-	g_signal_handlers_block_by_func(G_OBJECT(sv->priv->document), 
-									on_document_char_added, sv);
-	gtk_text_buffer_delete(GTK_TEXT_BUFFER(sv->priv->document), &begin, &end);
-	gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(sv->priv->document),
-									 completion, strlen(completion));
-	g_signal_handlers_unblock_by_func(G_OBJECT(sv->priv->document), 
-									on_document_char_added, sv);	
+	if (sv->priv->assist_tip)
+		gtk_widget_destroy (GTK_WIDGET (sv->priv->assist_tip));
 }
 
 static void
 iassist_iface_init(IAnjutaEditorAssistIface* iface)
 {
-	iface->add_trigger = iassist_add_trigger;
-	iface->remove_trigger = iassist_remove_trigger;
 	iface->suggest = iassist_suggest;
+	iface->hide_suggestions = iassist_hide_suggestions;
 	iface->get_suggestions = iassist_get_suggestions;
-	iface->tip = iassist_tip;
-	iface->react = iassist_react;
+	iface->show_tips = iassist_show_tips;
+	iface->cancel_tips = iassist_cancel_tips;
+}
+
+static gboolean
+isearch_forward (IAnjutaEditorSearch* isearch,
+				 const gchar* search,
+				 gboolean case_sensitive,
+				 IAnjutaEditorCell* istart, 
+				 IAnjutaEditorCell* iend,
+				 IAnjutaEditorCell** iresult_start,
+				 IAnjutaEditorCell** iresult_end,
+				 GError** e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW (isearch);
+	
+	SourceviewCell* start = SOURCEVIEW_CELL (istart);
+	SourceviewCell* end = SOURCEVIEW_CELL (iend);
+	
+	GtkTextIter* start_iter = sourceview_cell_get_iter (start);
+	GtkTextIter* end_iter = sourceview_cell_get_iter (end);
+	
+	GtkTextIter result_start, result_end;
+	
+	GtkSourceSearchFlags flags = 0;
+	
+	if (!case_sensitive)
+	{
+		flags = GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
+	}
+	
+	gboolean result = 
+		gtk_source_iter_forward_search (start_iter, search, flags, &result_start, &result_end,
+									end_iter);
+
+	if (result)
+	{
+		if (iresult_start)
+		{
+			*iresult_start = IANJUTA_EDITOR_CELL (sourceview_cell_new (&result_start,
+																	   GTK_TEXT_VIEW (sv->priv->view)));
+		}
+		if (iresult_end)
+		{
+			*iresult_end = IANJUTA_EDITOR_CELL (sourceview_cell_new (&result_end,
+																	 GTK_TEXT_VIEW (sv->priv->view)));			
+		}
+	}
+	
+	return result;
+}
+
+static gboolean
+isearch_backward (IAnjutaEditorSearch* isearch,
+				  const gchar* search,
+				  gboolean case_sensitive,
+				  IAnjutaEditorCell* istart, 
+				  IAnjutaEditorCell* iend,
+				  IAnjutaEditorCell** iresult_start,
+				  IAnjutaEditorCell** iresult_end,
+				  GError** e)
+{
+	Sourceview* sv = ANJUTA_SOURCEVIEW (isearch);
+	
+	SourceviewCell* start = SOURCEVIEW_CELL (istart);
+	SourceviewCell* end = SOURCEVIEW_CELL (iend);
+	
+	GtkTextIter* start_iter = sourceview_cell_get_iter (start);
+	GtkTextIter* end_iter = sourceview_cell_get_iter (end);
+	
+	GtkTextIter result_start, result_end;
+	
+	GtkSourceSearchFlags flags = 0;
+	
+	if (!case_sensitive)
+	{
+		flags = GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
+	}
+	
+	gboolean result = 
+		gtk_source_iter_backward_search (start_iter, search, flags, &result_start, &result_end,
+									end_iter);
+
+	if (result)
+	{
+		if (iresult_start)
+		{
+			*iresult_start = IANJUTA_EDITOR_CELL (sourceview_cell_new (&result_start,
+																	   GTK_TEXT_VIEW (sv->priv->view)));
+		}
+		if (iresult_end)
+		{
+			*iresult_end = IANJUTA_EDITOR_CELL (sourceview_cell_new (&result_end,
+																	 GTK_TEXT_VIEW (sv->priv->view)));			
+		}
+	}
+	
+	return result;
+}
+
+static void
+isearch_iface_init(IAnjutaEditorSearchIface* iface)
+{
+	iface->forward = isearch_forward;
+	iface->backward = isearch_backward;
 }
 
 ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_SCROLLED_WINDOW);
+ANJUTA_TYPE_ADD_INTERFACE(idocument, IANJUTA_TYPE_DOCUMENT);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
 ANJUTA_TYPE_ADD_INTERFACE(isavable, IANJUTA_TYPE_FILE_SAVABLE);
 ANJUTA_TYPE_ADD_INTERFACE(ieditor, IANJUTA_TYPE_EDITOR);
@@ -1928,4 +2107,5 @@ ANJUTA_TYPE_ADD_INTERFACE(iconvert, IANJUTA_TYPE_EDITOR_CONVERT);
 ANJUTA_TYPE_ADD_INTERFACE(ibookmark, IANJUTA_TYPE_BOOKMARK);
 ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
+ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
 ANJUTA_TYPE_END;
